import pytest

from insights.core.exceptions import ParseException
from insights.parsers import brctl_show
from insights.parsers.brctl_show import BrctlShow
from insights.tests import context_wrap

BRCTL_SHOW = """
bridge name     bridge id               STP enabled     interfaces
br0             8000.08002731ddfd       no              eth1
                                                        eth2
                                                        eth3
br1             8000.0800278cdb62       no              eth4
                                                        eth5
br2             8000.0800278cdb63       yes             eth6
docker0         8000.0242d4cf2112       no
""".strip()

BRCTL_SHOW_TAB = """
bridge name	bridge id		STP enabled	interfaces
br0		8000.2047478aa2e8	no		em1
							vnet0
							vnet1
virbr9		8000.525400263a23	yes		virbr9-nic
"""  # noqa: W191, E101

BRCTL_SHOW_NO_BRIDGES = """
bridge name     bridge id   STP enabled     interfaces

"""

BRCTL_SHOW_LESS_COLUMN = """
bridge name     bridge id

"""

BRCTL_SHOW_ERROR = """
/usr/sbin/brctl: file not found
""".strip()

BRCTL_SHOW_TIMEOUT = """
timeout: failed to run command `/usr/sbin/brctl':
""".strip()


def test_get_brctl_show():
    # the content is splitted with tab     # noqa: E101
    result1 = BrctlShow(context_wrap(BRCTL_SHOW_TAB)).group_by_iface
    assert result1["br0"] == {
                "bridge id": "8000.2047478aa2e8",
                "STP enabled": "no",
                "interfaces": ['em1', 'vnet0', 'vnet1']
            }

    # the content is split with space
    result = BrctlShow(context_wrap(BRCTL_SHOW)).group_by_iface
    assert len(result) == 4

    assert result["br0"] == {
                "bridge id": "8000.08002731ddfd",
                "STP enabled": "no",
                "interfaces": ['eth1', 'eth2', 'eth3']
            }
    assert result["br1"] == {
                "bridge id": "8000.0800278cdb62",
                "STP enabled": "no",
                "interfaces": ['eth4', 'eth5']
            }
    assert result["br2"] == {
                "bridge id": "8000.0800278cdb63",
                "STP enabled": "yes",
                "interfaces": ['eth6']
            }
    assert result["docker0"] == {
                "bridge id": "8000.0242d4cf2112",
                "STP enabled": "no"
            }

    # Test handling of system with no bridges
    with pytest.raises(ParseException) as e_info:
        brctl_show.BrctlShow(context_wrap(BRCTL_SHOW_ERROR))
    assert "Invalid Data Found" in str(e_info.value)

    # Test handling of error output
    with pytest.raises(ParseException) as e_info:
        brctl_show.BrctlShow(context_wrap(BRCTL_SHOW_LESS_COLUMN))
    assert "Invalid Data Found" in str(e_info.value)

    with pytest.raises(ParseException) as e_info:
        brctl_show.BrctlShow(context_wrap(BRCTL_SHOW_TIMEOUT))
    assert "Invalid Data Found" in str(e_info.value)



"""
Basic datasources for container specs
"""
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.parsers.docker_list import DockerListContainers
from insights.parsers.podman_list import PodmanListContainers
from insights.specs.datasources import DEFAULT_SHELL_TIMEOUT


@datasource([PodmanListContainers, DockerListContainers], HostContext, timeout=240)
def running_rhel_containers(broker):
    """
    Returns a list of tuple of (image, <podman|docker>, container_id) of the running
    containers.

    From RHEL 8, the "docker" command is from the package "podman-docker". In
    general when using command "docker" to access a container, the following
    message will be reported on the top::

        Emulate Docker CLI using podman. Create /etc/containers/nodocker to quiet msg.

    But sometimes, this line won't be outputted as expected, in this case, it's
    necessary to remove the duplicated containers from the output of "docker".
    """
    def _is_rhel_image(ctx, c_info):
        """Only collect the containers based from RHEL images"""
        try:
            ret = ctx.shell_out("/usr/bin/%s exec %s cat /etc/redhat-release" % c_info, timeout=DEFAULT_SHELL_TIMEOUT)
            if ret and len(ret) == 1 and "red hat enterprise linux" in ret[0].lower():
                return True
        except Exception:
            # return False when there is no such file "/etc/redhat-releas"
            pass
        return False

    cs = []
    podman_container = set()
    if (PodmanListContainers in broker):
        podman_c = broker[PodmanListContainers]
        for name in podman_c.running_containers:
            container_id = podman_c.containers[name]['CONTAINER ID']
            podman_container.add(container_id)
            c_info = ('podman', container_id[:12])
            if not _is_rhel_image(broker[HostContext], c_info):
                # skip containers from non-rhel image
                continue
            cs.append((podman_c.containers[name]['IMAGE'],) + c_info)
    if (DockerListContainers in broker):
        docker_c = broker[DockerListContainers]
        for name in docker_c.running_containers:
            container_id = docker_c.containers[name]['CONTAINER ID']
            c_info = ('docker', container_id[:12])
            if (container_id in podman_container or
                    not _is_rhel_image(broker[HostContext], c_info)):
                # skip containers from non-rhel image and
                # skip duplicated containers managed by "podman"
                continue
            cs.append((docker_c.containers[name]['IMAGE'],) + c_info)
    if cs:
        # Return list of tuple:
        # - (image, <podman|docker>, container_id)
        return cs

    raise SkipComponent



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import satellite_enabled_features
from insights.tests import context_wrap


enabled_features = '''
["ansible","dhcp","discovery",dynflow","logs","openscap","pulp","puppet","puppetca","ssh","templates","tftp"]
'''
empty_enabled_features = '''
[]
'''


def test_HTL_doc_examples():
    satellite_feature = satellite_enabled_features.SatelliteEnabledFeatures(context_wrap(enabled_features))
    globs = {
        'satellite_features': satellite_feature
    }
    failed, tested = doctest.testmod(satellite_enabled_features, globs=globs)
    assert failed == 0


def test_features_on_satellite():
    features = satellite_enabled_features.SatelliteEnabledFeatures(context_wrap(enabled_features))
    assert len(features) == 12
    assert features == ['ansible', 'dhcp', 'discovery', 'dynflow', 'logs', 'openscap', 'pulp', 'puppet', 'puppetca', 'ssh', 'templates', 'tftp']


def test_empty_features():
    with pytest.raises(SkipComponent):
        satellite_enabled_features.SatelliteEnabledFeatures(context_wrap(empty_enabled_features))



"""
IPA config - file ``/etc/ipa/default.conf``
===========================================
"""
try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

from insights.core import IniConfigFile, NoOptionError
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.ipa_default_conf)
class IPAConfig(IniConfigFile):
    """Parse the IPA default configuration

    The file ``/etc/ipa/default.conf`` contains settings for IPA client and
    server such as domain and realm name. It is a standard ini file. The
    ``global``  section must exist. It contains all IPA settings. Other
    sections are unused.

    The parser provides additional properties which implements the same
    fallbacks as IPA's internal config system. For example the ``server``
    property attempts to get the server name from ``[global] server``, then
    falls back to the net location of ``xmlrpc_uri`` and ``jsonrpc_uri``.

    Raises:
        ParseException: When config is missing required section or options.

    Sample configuration::
        [global]
        host = client.ipa.test
        basedn = dc=ipa,dc=test
        realm = IPA.TEST
        domain = ipa.test
        xmlrpc_uri = https://server.ipa.test/ipa/xml

    Examples:
        >>> type(ipaconfig)
        <class 'insights.parsers.ipa_conf.IPAConfig'>
        >>> ipaconfig.ipa_section
        'global'
        >>> ipaconfig.sections()
        ['global']
        >>> ipaconfig.server
        'server.ipa.test'
        >>> ipaconfig.domain
        'ipa.test'
        >>> ipaconfig.realm
        'IPA.TEST'
        >>> ipaconfig.basedn
        'dc=ipa,dc=test'
        >>> ipaconfig.xmlrpc_uri
        'https://server.ipa.test/ipa/xml'
        >>> ipaconfig.jsonrpc_uri
        'https://server.ipa.test/ipa/json'

    """

    ipa_section = "global"
    _server = None

    def parse_content(self, content, allow_no_value=False):
        super(IPAConfig, self).parse_content(content, allow_no_value)
        # validate configuration
        if self.ipa_section not in self:
            raise ParseException(
                "Missing '{0}' section".format(self.ipa_section)
            )
        try:
            if not self.realm:
                raise ValueError("invalid realm")
            if not self.server:
                raise ValueError("invalid server")
        except ValueError as e:
            raise ParseException(str(e))

    @property
    def server(self):
        """IPA server FQDN

        Falls back to ``xmlrpc_uri`` and ``jsonrpc_uri``
        """
        if self._server is not None:
            return self._server

        try:
            server = self.get(self.ipa_section, "server")
        except NoOptionError:
            # fall back
            try:
                uri = self.get(self.ipa_section, "xmlrpc_uri")
            except NoOptionError:
                try:
                    uri = self.get(self.ipa_section, "jsonrpc_uri")
                except NoOptionError:
                    raise ValueError(
                        "server, xmlrpc_uri, and jsonrpc_uri missing"
                    )
            server = urlparse(uri).netloc

        self._server = server
        return server

    @property
    def realm(self):
        """Kerberos realm name"""
        try:
            return self.get(self.ipa_section, "realm")
        except NoOptionError:
            # no fallback
            raise ValueError("realm missing")

    @property
    def domain(self):
        """Domain name

        Falls back to lower-case Kerberos ``realm`` name
        """
        try:
            return self.get(self.ipa_section, "domain")
        except NoOptionError:
            # fall back to lower realm name
            return self.realm.lower()

    @property
    def basedn(self):
        """LDAP base DN

        Falls back to basedn from ``domain``'s domain components
        """
        try:
            return self.get(self.ipa_section, "basedn")
        except NoOptionError:
            parts = self.domain.split(".")
            return ",".join("dc={0}".format(part) for part in parts)

    @property
    def ldap_uri(self):
        """LDAP server uri

        Falls back to ``server`` to build an ``ldap://`` URI.
        """
        try:
            return self.get(self.ipa_section, "ldap_uri")
        except NoOptionError:
            return "ldap://{0}".format(self.server)

    @property
    def xmlrpc_uri(self):
        """XML-RPC uri

        Falls back to ``server`` to build an ``https://`` URI.
        """
        try:
            return self.get(self.ipa_section, "xmlrpc_uri")
        except NoOptionError:
            return "https://{0}/ipa/xml".format(self.server)

    @property
    def jsonrpc_uri(self):
        """JSON-RPC uri

        Falls back to ``server`` to build an ``https://`` URI.
        """
        try:
            return self.get(self.ipa_section, "jsonrpc_uri")
        except NoOptionError:
            return "https://{0}/ipa/json".format(self.server)



import doctest

from insights.parsers import ls_lib_firmware
from insights.parsers.ls_lib_firmware import LsLibFW
from insights.tests import context_wrap

LS_LIB_FW = """
/lib/firmware:
total 37592
drwxr-xr-x. 83 0 0    8192 Aug 14 02:43 .
dr-xr-xr-x. 26 0 0    4096 Aug 14 02:22 ..
drwxr-xr-x.  2 0 0      40 Aug 14 02:42 3com
lrwxrwxrwx.  1 0 0      16 Aug 14 02:42 a300_pfp.fw -> qcom/a300_pfp.fw
lrwxrwxrwx.  1 0 0      16 Aug 14 02:42 a300_pm4.fw -> qcom/a300_pm4.fw
drwxr-xr-x.  2 0 0      34 Aug 14 02:42 acenic
drwxr-xr-x.  2 0 0      50 Aug 14 02:42 adaptec
drwxr-xr-x.  2 0 0      73 Aug 14 02:42 advansys

/lib/firmware/3com:
total 84
drwxr-xr-x.  2 0 0    40 Aug 14 02:42 .
drwxr-xr-x. 83 0 0  8192 Aug 14 02:43 ..
-rw-r--r--.  1 0 0 24880 Jun  6 10:14 3C359.bin
-rw-r--r--.  1 0 0 44548 Jun  6 10:14 typhoon.bin

/lib/firmware/acenic:
total 160
drwxr-xr-x.  2 0 0    34 Aug 14 02:42 .
drwxr-xr-x. 83 0 0  8192 Aug 14 02:43 ..
-rw-r--r--.  1 0 0 73116 Jun  6 10:14 tg1.bin
-rw-r--r--.  1 0 0 77452 Jun  6 10:14 tg2.bin

/lib/firmware/adaptec:
total 20
drwxr-xr-x.  2 0 0   50 Aug 14 02:42 .
drwxr-xr-x. 83 0 0 8192 Aug 14 02:43 ..
-rw-r--r--.  1 0 0  832 Jun  6 10:14 starfire_rx.bin
-rw-r--r--.  1 0 0  832 Jun  6 10:14 starfire_tx.bin

/lib/firmware/advansys:
total 40
drwxr-xr-x.  2 0 0   73 Aug 14 02:42 .
drwxr-xr-x. 83 0 0 8192 Aug 14 02:43 ..
-rw-r--r--.  1 0 0 5026 Jun  6 10:14 3550.bin
-rw-r--r--.  1 0 0 5340 Jun  6 10:14 38C0800.bin
-rw-r--r--.  1 0 0 6334 Jun  6 10:14 38C1600.bin
-rw-r--r--.  1 0 0 2308 Jun  6 10:14 mcode.bin

/lib/firmware/bnx2:
total 1448
drwxr-xr-x.  2 0 0   4096 Aug 14 02:43 .
drwxr-xr-x. 83 0 0   8192 Aug 14 02:43 ..
-rw-r--r--.  1 0 0  92628 Jun  6 10:14 bnx2-mips-06-4.6.16.fw
-rw-r--r--.  1 0 0  93172 Jun  6 10:14 bnx2-mips-06-5.0.0.j3.fw
-rw-r--r--.  1 0 0  94252 Jun  6 10:14 bnx2-mips-06-5.0.0.j6.fw
-rw-r--r--.  1 0 0  92824 Jun  6 10:14 bnx2-mips-06-6.2.3.fw
-rw-r--r--.  1 0 0  92760 Jun  6 10:14 bnx2-mips-09-4.6.17.fw
-rw-r--r--.  1 0 0  96996 Jun  6 10:14 bnx2-mips-09-5.0.0.j15.fw

/lib/firmware/bnx2x:
total 8772
drwxr-xr-x.  2 0 0   4096 Aug 14 02:43 .
drwxr-xr-x. 83 0 0   8192 Aug 14 02:43 ..
-rw-r--r--.  1 0 0 151568 Jun  6 10:14 bnx2x-e1-6.0.34.0.fw
-rw-r--r--.  1 0 0 151680 Jun  6 10:14 bnx2x-e1-6.2.5.0.fw
-rw-r--r--.  1 0 0 151688 Jun  6 10:14 bnx2x-e1-6.2.9.0.fw
-rw-r--r--.  1 0 0 161144 Jun  6 10:14 bnx2x-e1-7.0.20.0.fw
-rw-r--r--.  1 0 0 161248 Jun  6 10:14 bnx2x-e1-7.0.23.0.fw
""".strip()


def test_ls_lib_firmware():
    lslib = LsLibFW(context_wrap(LS_LIB_FW))
    assert "bnx2x" not in lslib
    assert "/lib/firmware/bnx2" in lslib
    assert "/lib/firmware/bnx2x" in lslib
    assert lslib.dirs_of("/lib/firmware") == ['.', '..', '3com', 'acenic', 'adaptec', 'advansys']
    assert lslib.files_of("/lib/firmware/bnx2x") == ['bnx2x-e1-6.0.34.0.fw', 'bnx2x-e1-6.2.5.0.fw', 'bnx2x-e1-6.2.9.0.fw', 'bnx2x-e1-7.0.20.0.fw', 'bnx2x-e1-7.0.23.0.fw']
    assert "bnx2x-e1-6.0.34.0.fw" in lslib.files_of("/lib/firmware/bnx2x")
    assert lslib.dir_contains("/lib/firmware/bnx2x", "bnx2x-e1-6.0.34.0.fw") is True
    assert lslib.total_of("/lib/firmware") == 37592


def test_ls_lib_firmware_doc_examples():
    env = {
        'lslibfw': LsLibFW(context_wrap(LS_LIB_FW)),
    }
    failed, total = doctest.testmod(ls_lib_firmware, globs=env)
    assert failed == 0



"""
Lists ALL the firmware packages
===============================

Parsers included in this module are:

LsLibFW - command ``/bin/ls -lanR /lib/firmware``
----------------------------------------------------

"""
from .. import parser, CommandParser, FileListing
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.ls_lib_firmware)
class LsLibFW(CommandParser, FileListing):
    """
    .. warning::
        This class is deprecated and will be removed from 3.5.0.
        Please use the :class:`insights.parsers.ls.LSlanR` instead.

    This parser will help to parse the output of command ``/bin/ls -lanR /lib/firmware``

    Typical output of the ``/bin/ls -lanR /lib/firmware`` command is::

    \t/lib/firmware:
    \ttotal 37592
    \tdrwxr-xr-x. 83 0 0    8192 Aug 14 02:43 .
    \tdr-xr-xr-x. 26 0 0    4096 Aug 14 02:22 ..
    \tdrwxr-xr-x.  2 0 0      40 Aug 14 02:42 3com
    \tlrwxrwxrwx.  1 0 0      16 Aug 14 02:42 a300_pfp.fw -> qcom/a300_pfp.fw
    \tlrwxrwxrwx.  1 0 0      16 Aug 14 02:42 a300_pm4.fw -> qcom/a300_pm4.fw
    \tdrwxr-xr-x.  2 0 0      34 Aug 14 02:42 acenic
    \tdrwxr-xr-x.  2 0 0      50 Aug 14 02:42 adaptec
    \tdrwxr-xr-x.  2 0 0      73 Aug 14 02:42 advansys
    \t
    \t/lib/firmware/3com:
    \ttotal 84
    \tdrwxr-xr-x.  2 0 0    40 Aug 14 02:42 .
    \tdrwxr-xr-x. 83 0 0  8192 Aug 14 02:43 ..
    \t-rw-r--r--.  1 0 0 24880 Jun  6 10:14 3C359.bin
    \t-rw-r--r--.  1 0 0 44548 Jun  6 10:14 typhoon.bin
    \t
    \t/lib/firmware/acenic:
    \ttotal 160
    \tdrwxr-xr-x.  2 0 0    34 Aug 14 02:42 .
    \tdrwxr-xr-x. 83 0 0  8192 Aug 14 02:43 ..
    \t-rw-r--r--.  1 0 0 73116 Jun  6 10:14 tg1.bin
    \t-rw-r--r--.  1 0 0 77452 Jun  6 10:14 tg2.bin

    Example:

        >>> type(lslibfw)
        <class 'insights.parsers.ls_lib_firmware.LsLibFW'>
        >>> lslibfw.files_of("/lib/firmware/bnx2x")
        ['bnx2x-e1-6.0.34.0.fw', 'bnx2x-e1-6.2.5.0.fw', 'bnx2x-e1-6.2.9.0.fw', 'bnx2x-e1-7.0.20.0.fw', 'bnx2x-e1-7.0.23.0.fw']
        >>> lslibfw.dir_contains("/lib/firmware/bnx2x", "bnx2x-e1-6.0.34.0.fw")
        True
        >>> lslibfw.dirs_of("/lib/firmware")
        ['.', '..', '3com', 'acenic', 'adaptec', 'advansys']
        >>> lslibfw.total_of("/lib/firmware")
        37592
    """
    def __init__(self, *args, **kwargs):
        deprecated(LsLibFW, "Please use the :class:`insights.parsers.ls.LSlanR` instead.", "3.5.0")
        super(LsLibFW, self).__init__(*args, **kwargs)



from insights.core.exceptions import ParseException
from insights.parsers import ls_rsyslog_errorfile
from insights.parsers.ls_rsyslog_errorfile import LsRsyslogErrorfile
from insights.tests import context_wrap
import doctest
import pytest

LS_RSYSLOG_ERRORFILE = """
/bin/ls: cannot access '/var/log/rsyslog/es-errors2.log': No such file or directory
/bin/ls: cannot access '/var/log/rsyslog/es-errors3.log': No such file or directory
-rw-r--r--. 1 0 0   9 Mar 15 17:16 /var/log/omelasticsearch.log
-rw-r--r--. 1 0 0 176 Mar 22 15:10 /var/log/rsyslog/es-errors1.log
"""

LS_RSYSLOG_ERRORFILE_ERROR = """
/bin/ls: cannot access '/var/log/omelasticsearch.log': No such file or directory
/bin/ls: cannot access '/var/log/rsyslog/es-errors1.log': No such file or directory
/bin/ls: cannot access '/var/log/rsyslog/es-errors2.log': No such file or directory
/bin/ls: cannot access '/var/log/rsyslog/es-errors3.log': No such file or directory
"""


def test_ls_rsyslog_errorfile():
    rsyslog_errorfile = LsRsyslogErrorfile(context_wrap(LS_RSYSLOG_ERRORFILE))
    assert len(rsyslog_errorfile.entries) == 2
    assert rsyslog_errorfile.entries.get('/var/log/omelasticsearch.log') == {'type': '-', 'perms': 'rw-r--r--.', 'links': 1, 'owner': '0', 'group': '0', 'size': 9, 'date': 'Mar 15 17:16', 'name': '/var/log/omelasticsearch.log', 'raw_entry': '-rw-r--r--. 1 0 0   9 Mar 15 17:16 /var/log/omelasticsearch.log', 'dir': ''}
    assert rsyslog_errorfile.entries.get('/var/log/rsyslog/es-errors1.log').get('size') == 176


def test_ls_rsyslog_errorfile_err():
    with pytest.raises(ParseException) as pe:
        LsRsyslogErrorfile(context_wrap(LS_RSYSLOG_ERRORFILE_ERROR))
        assert 'Error:' in str(pe)


def test_ls_rsyslog_errorfile_doc_examples():
    env = {
        'rsyslog_errorfile': LsRsyslogErrorfile(context_wrap(LS_RSYSLOG_ERRORFILE)),
    }
    failed, total = doctest.testmod(ls_rsyslog_errorfile, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import virt_who_conf
from insights.tests import context_wrap

VWHO_CONF = """
## This is a template for virt-who global configuration files. Please see
## virt-who-config(5) manual page for detailed information.
##
## virt-who checks /etc/virt-who.conf for sections 'global' and 'defaults'.
## The sections and their values are explained below.
## NOTE: These sections retain their special meaning and function only when present in /etc/virt-who.conf
##
## You can uncomment and fill following template or create new file with
## similar content.

#Terse version of the general config template:
[global]

interval=3600
#reporter_id=
debug=False
oneshot=False
#log_per_config=False
#log_dir=
#log_file=
#configs=

[defaults]
owner=Satellite
env=Satellite
hypervisor_id=ID1
"""


def test_doc_examples():
    failed_count, tests = doctest.testmod(
        virt_who_conf,
        globs={'conf': virt_who_conf.VirtWhoConf(context_wrap(VWHO_CONF))}
    )
    assert failed_count == 0


def test_virt_who_conf_empty():
    with pytest.raises(SkipComponent):
        assert virt_who_conf.VirtWhoConf(context_wrap('')) is None


def test_virt_who_conf():
    vwho_conf = virt_who_conf.VirtWhoConf(context_wrap(VWHO_CONF))
    assert vwho_conf.has_option('global', 'debug')
    assert vwho_conf.get('global', 'oneshot') == "False"
    assert vwho_conf.getboolean('global', 'oneshot') is False
    assert vwho_conf.get('global', 'interval') == "3600"
    assert vwho_conf.getint('global', 'interval') == 3600
    assert vwho_conf.items('defaults') == {
            'owner': 'Satellite',
            'env': 'Satellite',
            'hypervisor_id': 'ID1'}



import json
import pytest

from insights.core import plugins
from insights.core.exceptions import ValidationException


def test_validate_good_response():
    assert plugins.make_response("a_test", foo="bar") == {
        "type": "rule",
        "error_key": "a_test",
        "foo": "bar"
    }
    assert plugins.make_fail("a_test", foo="bar") == {
        "type": "rule",
        "error_key": "a_test",
        "foo": "bar"
    }
    assert plugins.make_pass("a_test", foo="bar") == {
        "type": "pass",
        "pass_key": "a_test",
        "foo": "bar"
    }
    assert plugins.make_fingerprint("a_test", foo="bar") == {
        "type": "fingerprint",
        "fingerprint_key": "a_test",
        "foo": "bar"
    }
    assert plugins.make_metadata(foo="bar") == {
        "type": "metadata",
        "foo": "bar"
    }
    assert plugins.make_metadata_key("foo", "bar") == {
        "type": "metadata_key",
        "key": "foo",
        "value": "bar"
    }


def test_disallow_invalid_keys():
    for bad in [[], None, set(), "", 1, lambda x: x]:
        with pytest.raises(ValidationException):
            plugins.make_response(bad)
        with pytest.raises(ValidationException):
            plugins.make_metadata_key(bad, "foo")


def test_disallow_type_key():
    with pytest.raises(ValidationException):
        plugins.make_response("foo", type="dance off")


def test_missing_error_key():
    with pytest.raises(ValidationException):
        plugins.make_response(None, foo="bar")
    with pytest.raises(ValidationException):
        plugins.make_metadata_key(None, "foo")


def test_response_too_big():
    content = "foo" * 50000
    assert plugins.make_response("TESTING", big=content) == {
        "type": "rule",
        "error_key": "TESTING",
        "max_detail_length_error": len(json.dumps({"error_key": "TESTING", "type": "rule", "big": content}))
    }


def test_str_without_type():
    d = plugins.make_response("TESTING", foo="bar")
    del d["type"]
    str(d)
    assert True



"""
IPCS Semaphores
===============

Combiner for parsing all semaphores. It uses the results of the
``IpcsS`` and ``IpcsSI`` parsers to collect complete semaphore information,and
use ``PsAuxcww`` parsers to determine if one semaphore is orphan.

"""

from insights.core.plugins import combiner
from insights.parsers.ps import PsAuxcww
from insights.parsers.ipcs import IpcsS, IpcsSI


class IpcsSemaphore(object):
    """
    Class for holding information about one semaphore.

    """
    def __init__(self, data):
        self.semid = None
        """str: Semaphore ID."""
        self.owner = None
        """str: Owner of the semaphore."""
        self.is_orphan = False
        """bool: Is it an orphan semaphore?"""
        self.pid_list = []
        """list: List of the related PID."""

        for k, v in data.items():
            setattr(self, k, v)


@combiner(IpcsS, IpcsSI, PsAuxcww)
class IpcsSemaphores(object):
    """
    Class for parsing all semaphores. Will generate IpcsSemaphore objects for
    each semaphores.

    Below is the logic to determine if semaphore an orphan::

    - PID=0 does not included in the related PID
    - Related PID cannot be found in running PIDs

    Examples:
        >>> oph_sem.count_of_all_sems()
        5
        >>> oph_sem.count_of_all_sems(owner='apache')
        4
        >>> oph_sem.count_of_orphan_sems()
        2
        >>> oph_sem.count_of_orphan_sems('apache')
        2
        >>> oph_sem.get_sem('65536').is_orphan
        False

    """
    def __init__(self, sem_s, sem_si, ps):
        pids = ps.running_pids()
        self._all_sems = {}
        self._orphan_sems = []
        for sem in sem_si:
            semid = sem.semid
            pid_list = sem.pid_list
            data = sem_s.get(semid)
            data['semid'] = semid
            data['pid_list'] = pid_list
            # check if it is orphan
            is_orphan = False
            if '0' not in pid_list and all(p not in pids for p in pid_list):
                is_orphan = True
            data['is_orphan'] = is_orphan
            sem_obj = IpcsSemaphore(data)
            self._all_sems[semid] = sem_obj
            if is_orphan:
                self._orphan_sems.append(sem_obj)

    def count_of_all_sems(self, owner=None):
        """
        Return the count of all semaphores by default, when ``owner`` is
        provided return the count of semaphores belong to ``owner``.

        Parameters:
            owner(str): Owner of semaphores.

        Returns:
            (int): the count of semaphores.
        """
        if owner:
            cnt = 0
            for sem in self._all_sems.values():
                cnt += 1 if sem.owner == owner else 0
            return cnt
        return len(self._all_sems)

    def count_of_orphan_sems(self, owner=None):
        """
        Return the count of orphan semaphores by default, when ``owner`` is
        provided return the count of orphan semaphores belong to ``owner``.

        Parameters:
            owner(str): Owner of semaphores.

        Returns:
            (int): the count of orphan semaphores
        """
        if owner:
            cnt = 0
            for sem in self._orphan_sems:
                cnt += 1 if sem.owner == owner else 0
            return cnt
        return len(self._orphan_sems)

    def orphan_sems(self, owner=None):
        """
        Return all the orphan semaphores by default, when ``owner`` is
        provided return the orphan semaphores belong to ``owner``.

        Parameters:
            owner(str): Owner of semaphores.

        Returns:
            (list): the ID list of orphan semaphores
        """
        orphans = []
        if owner:
            for sem in self._orphan_sems:
                if sem.owner == owner:
                    orphans.append(sem.semid)
            return orphans
        return [sem.semid for sem in self._orphan_sems]

    def get_sem(self, semid):
        """
        Return an IpcsSemaphore instance which semid is ``semid``

        Returns:
            (IpcsSemaphore): the instance of IpcsSemaphore
        """
        return self._all_sems.get(semid)

    def __iter__(self):
        """
        (iterable): Iterate the semaphores in no order.
        """
        for sem in self._all_sems.values():
            yield sem



from insights.parsers.branch_info import BranchInfo
from insights.tests import context_wrap

bi_conf_content = """
{"remote_branch": -1, "remote_leaf": -1}
""".strip()


def test_settings_yml():
    ctx = context_wrap(bi_conf_content)
    ctx.content = bi_conf_content
    result = BranchInfo(ctx)
    assert result.data['remote_branch'] == -1
    assert result.data['remote_leaf'] == -1



from insights.parsers.ceph_insights import CephInsights
from insights.parsers import ceph_insights
from insights.tests import context_wrap
import six
import doctest

TEST_DATA = """
{
  "crush_map": {
    "tunables": {
      "profile": "jewel",
      "minimum_required_version": "jewel",
      "has_v3_rules": 0,
      "has_v4_buckets": 1,
      "choose_total_tries": 50,
      "require_feature_tunables3": 1,
      "require_feature_tunables5": 1,
      "legacy_tunables": 0,
      "chooseleaf_descend_once": 1,
      "chooseleaf_stable": 1,
      "choose_local_fallback_tries": 0,
      "has_v2_rules": 0,
      "straw_calc_version": 1,
      "allowed_bucket_algs": 54,
      "has_v5_rules": 0,
      "require_feature_tunables2": 1,
      "optimal_tunables": 1,
      "choose_local_tries": 0,
      "chooseleaf_vary_r": 1,
      "require_feature_tunables": 1
    },
    "rules": [
      {
        "min_size": 1,
        "rule_name": "replicated_rule",
        "steps": [
          {
            "item_name": "default",
            "item": -1,
            "op": "take"
          },
          {
            "num": 0,
            "type": "osd",
            "op": "choose_firstn"
          },
          {
            "op": "emit"
          }
        ],
        "ruleset": 0,
        "type": 1,
        "rule_id": 0,
        "max_size": 10
      }
    ],
    "buckets": [
      {
        "hash": "rjenkins1",
        "name": "default",
        "weight": 1926,
        "type_id": 10,
        "alg": "straw2",
        "type_name": "root",
        "items": [
          {
            "id": -3,
            "weight": 1926,
            "pos": 0
          }
        ],
        "id": -1
      },
      {
        "hash": "rjenkins1",
        "name": "default~ssd",
        "weight": 1926,
        "type_id": 10,
        "alg": "straw2",
        "type_name": "root",
        "items": [
          {
            "id": -4,
            "weight": 1926,
            "pos": 0
          }
        ],
        "id": -2
      },
      {
        "hash": "rjenkins1",
        "name": "daq",
        "weight": 1926,
        "type_id": 1,
        "alg": "straw2",
        "type_name": "host",
        "items": [
          {
            "id": 0,
            "weight": 642,
            "pos": 0
          },
          {
            "id": 1,
            "weight": 642,
            "pos": 1
          },
          {
            "id": 2,
            "weight": 642,
            "pos": 2
          }
        ],
        "id": -3
      },
      {
        "hash": "rjenkins1",
        "name": "daq~ssd",
        "weight": 1926,
        "type_id": 1,
        "alg": "straw2",
        "type_name": "host",
        "items": [
          {
            "id": 0,
            "weight": 642,
            "pos": 0
          },
          {
            "id": 1,
            "weight": 642,
            "pos": 1
          },
          {
            "id": 2,
            "weight": 642,
            "pos": 2
          }
        ],
        "id": -4
      }
    ],
    "devices": [
      {
        "class": "ssd",
        "id": 0,
        "name": "osd.0"
      },
      {
        "class": "ssd",
        "id": 1,
        "name": "osd.1"
      },
      {
        "class": "ssd",
        "id": 2,
        "name": "osd.2"
      }
    ],
    "choose_args": {},
    "types": [
      {
        "name": "osd",
        "type_id": 0
      },
      {
        "name": "host",
        "type_id": 1
      },
      {
        "name": "chassis",
        "type_id": 2
      },
      {
        "name": "rack",
        "type_id": 3
      },
      {
        "name": "row",
        "type_id": 4
      },
      {
        "name": "pdu",
        "type_id": 5
      },
      {
        "name": "pod",
        "type_id": 6
      },
      {
        "name": "room",
        "type_id": 7
      },
      {
        "name": "datacenter",
        "type_id": 8
      },
      {
        "name": "region",
        "type_id": 9
      },
      {
        "name": "root",
        "type_id": 10
      }
    ]
  },
  "pg_summary": {
    "by_osd": {
      "1": {
        "peering": 13
      },
      "0": {
        "peering": 16
      },
      "2": {
        "peering": 16
      }
    },
    "by_pool": {
      "1": {
        "peering": 8
      },
      "2": {
        "peering": 8
      }
    },
    "all": {
      "peering": 16
    },
    "pg_stats_sum": {
      "acting": 45,
      "log_size": 39,
      "ondisk_log_size": 39,
      "stat_sum": {
        "num_evict": 0,
        "num_evict_kb": 0,
        "num_bytes_hit_set_archive": 0,
        "num_whiteouts": 0,
        "num_objects_pinned": 0,
        "num_scrub_errors": 0,
        "num_evict_mode_full": 0,
        "num_read": 0,
        "num_objects_recovered": 0,
        "num_objects_omap": 14,
        "num_objects_missing_on_primary": 0,
        "num_write": 45,
        "num_object_clones": 0,
        "num_objects": 22,
        "num_deep_scrub_errors": 0,
        "num_shallow_scrub_errors": 0,
        "num_read_kb": 0,
        "num_objects_missing": 0,
        "num_flush_kb": 0,
        "num_flush_mode_high": 0,
        "num_write_kb": 13,
        "num_large_omap_objects": 0,
        "num_evict_mode_some": 0,
        "num_objects_degraded": 0,
        "num_flush": 0,
        "num_objects_misplaced": 0,
        "num_bytes_recovered": 0,
        "num_objects_hit_set_archive": 0,
        "num_legacy_snapsets": 0,
        "num_keys_recovered": 0,
        "num_flush_mode_low": 0,
        "num_objects_manifest": 0,
        "num_objects_unfound": 0,
        "num_promote": 0,
        "num_object_copies": 66,
        "num_bytes": 2286,
        "num_objects_dirty": 22
      },
      "up": 45
    }
  },
  "service_map": {
    "services": {},
    "epoch": 1,
    "modified": "0.000000"
  },
  "crashes": {
    "hours": 24,
    "summary": {}
  },
  "mon_status": {
    "election_epoch": 12,
    "quorum_age": "11.1322s",
    "outside_quorum": [],
    "rank": 0,
    "monmap": {
      "features": {
        "optional": [],
        "persistent": [
          "kraken",
          "luminous",
          "mimic",
          "osdmap-prune",
          "nautilus"
        ]
      },
      "created": "2018-09-21 10:41:49.406653",
      "modified": "2018-09-21 10:41:49.406653",
      "epoch": 1,
      "mons": [
        {
          "public_addr": "127.0.0.1:40226/0",
          "name": "a",
          "rank": 0,
          "addr": "127.0.0.1:40226/0"
        },
        {
          "public_addr": "127.0.0.1:40227/0",
          "name": "b",
          "rank": 1,
          "addr": "127.0.0.1:40227/0"
        },
        {
          "public_addr": "127.0.0.1:40228/0",
          "name": "c",
          "rank": 2,
          "addr": "127.0.0.1:40228/0"
        }
      ],
      "fsid": "d02162fd-bee9-4cb5-973b-2f7b6d97102a"
    },
    "state": "leader",
    "features": {
      "quorum_con": "4611087854031142911",
      "quorum_mon": [
        "kraken",
        "luminous",
        "mimic",
        "osdmap-prune",
        "nautilus"
      ],
      "required_mon": [
        "kraken",
        "luminous",
        "mimic",
        "osdmap-prune",
        "nautilus"
      ],
      "required_con": "2449958747315912708"
    },
    "extra_probe_peers": [],
    "feature_map": {
      "mds": [
        {
          "release": "luminous",
          "num": 1,
          "features": "0x3ffddff8ffa4ffff"
        }
      ],
      "mon": [
        {
          "release": "luminous",
          "num": 1,
          "features": "0x3ffddff8ffa4ffff"
        }
      ],
      "mgr": [
        {
          "release": "luminous",
          "num": 1,
          "features": "0x3ffddff8ffa4ffff"
        }
      ]
    },
    "quorum": [
      0,
      1,
      2
    ],
    "sync_provider": [],
    "name": "a"
  },
  "df": {
    "pools": [
      {
        "stats": {
          "wr": 0,
          "quota_objects": 0,
          "bytes_used": 0,
          "max_avail": 9553910784,
          "rd": 0,
          "rd_bytes": 0,
          "objects": 0,
          "percent_used": 0.0,
          "kb_used": 0,
          "quota_bytes": 0,
          "raw_bytes_used": 0,
          "wr_bytes": 0,
          "dirty": 0
        },
        "name": "cephfs_data_a",
        "id": 1
      },
      {
        "stats": {
          "wr": 45,
          "quota_objects": 0,
          "bytes_used": 2286,
          "max_avail": 9553910784,
          "rd": 0,
          "rd_bytes": 0,
          "objects": 22,
          "percent_used": 2.3927370307319507e-07,
          "kb_used": 3,
          "quota_bytes": 0,
          "raw_bytes_used": 6858,
          "wr_bytes": 13312,
          "dirty": 22
        },
        "name": "cephfs_metadata_a",
        "id": 2
      }
    ],
    "stats": {
      "total_objects": 22,
      "total_used_bytes": 3425656832,
      "total_bytes": 32413556736,
      "total_percent_used": 10.56859302520752,
      "total_avail_bytes": 28987899904
    }
  },
  "osd_tree": {
    "nodes": [
      {
        "children": [
          -3
        ],
        "type_id": 10,
        "type": "root",
        "id": -1,
        "name": "default"
      },
      {
        "name": "daq",
        "type_id": 1,
        "id": -3,
        "pool_weights": {},
        "type": "host",
        "children": [
          2,
          1,
          0
        ]
      },
      {
        "status": "up",
        "name": "osd.0",
        "exists": 1,
        "type_id": 0,
        "reweight": 1.0,
        "crush_weight": 0.009796142578125,
        "pool_weights": {},
        "primary_affinity": 1.0,
        "depth": 2,
        "device_class": "ssd",
        "type": "osd",
        "id": 0
      },
      {
        "status": "up",
        "name": "osd.1",
        "exists": 1,
        "type_id": 0,
        "reweight": 1.0,
        "crush_weight": 0.009796142578125,
        "pool_weights": {},
        "primary_affinity": 1.0,
        "depth": 2,
        "device_class": "ssd",
        "type": "osd",
        "id": 1
      },
      {
        "status": "up",
        "name": "osd.2",
        "exists": 1,
        "type_id": 0,
        "reweight": 1.0,
        "crush_weight": 0.009796142578125,
        "pool_weights": {},
        "primary_affinity": 1.0,
        "depth": 2,
        "device_class": "ssd",
        "type": "osd",
        "id": 2
      }
    ],
    "stray": []
  },
  "osd_metadata": {
    "1": {
      "bluefs_db_size": "67108864",
      "bluestore_bdev_size": "10737418240",
      "bluestore_bdev_driver": "KernelDevice"
    },
    "0": {
      "bluefs_db_size": "67108864",
      "bluestore_bdev_size": "10737418240",
      "bluestore_bdev_driver": "KernelDevice"
    },
    "2": {
      "bluefs_db_size": "67108864",
      "bluestore_bdev_size": "10737418240",
      "bluestore_bdev_driver": "KernelDevice"
    }
  },
  "osd_dump": {
    "nearfull_ratio": 0.9900000095367432,
    "backfillfull_ratio": 0.9900000095367432,
    "last_in_change": "2018-09-21 10:42:23.201021",
    "full_ratio": 0.9900000095367432,
    "cluster_snapshot": "",
    "pg_upmap": [],
    "new_purged_snaps": [],
    "erasure_code_profiles": {
      "default": {
        "crush-failure-domain": "osd",
        "k": "2",
        "technique": "reed_sol_van",
        "m": "1",
        "plugin": "jerasure"
      }
    },
    "osds": [
      {
        "public_addrs": {
          "addrvec": [
            {
              "nonce": 4283,
              "type": "legacy",
              "addr": "127.0.0.1:6800"
            }
          ]
        },
        "weight": 1.0,
        "up_from": 17,
        "heartbeat_front_addr": {
          "addrvec": [
            {
              "nonce": 4283,
              "type": "legacy",
              "addr": "127.0.0.1:6802"
            }
          ]
        },
        "down_at": 16,
        "up": 1,
        "cluster_addrs": {
          "addrvec": [
            {
              "nonce": 4283,
              "type": "legacy",
              "addr": "127.0.0.1:6801"
            }
          ]
        },
        "lost_at": 0,
        "primary_affinity": 1.0,
        "state": [
          "exists",
          "up"
        ],
        "last_clean_begin": 5,
        "last_clean_end": 15,
        "in": 1,
        "public_addr": "127.0.0.1:6800/4283",
        "up_thru": 19,
        "heartbeat_back_addrs": {
          "addrvec": [
            {
              "nonce": 4283,
              "type": "legacy",
              "addr": "127.0.0.1:6803"
            }
          ]
        },
        "cluster_addr": "127.0.0.1:6801/4283",
        "osd": 0,
        "uuid": "c8c26eda-d1e4-4ab2-839d-a07573414e41"
      },
      {
        "public_addrs": {
          "addrvec": [
            {
              "nonce": 4559,
              "type": "legacy",
              "addr": "127.0.0.1:6804"
            }
          ]
        },
        "weight": 1.0,
        "up_from": 18,
        "heartbeat_front_addr": {
          "addrvec": [
            {
              "nonce": 4559,
              "type": "legacy",
              "addr": "127.0.0.1:6806"
            }
          ]
        },
        "down_at": 17,
        "up": 1,
        "cluster_addrs": {
          "addrvec": [
            {
              "nonce": 4559,
              "type": "legacy",
              "addr": "127.0.0.1:6805"
            }
          ]
        },
        "lost_at": 0,
        "primary_affinity": 1.0,
        "state": [
          "exists",
          "up"
        ],
        "last_clean_begin": 8,
        "last_clean_end": 15,
        "in": 1,
        "public_addr": "127.0.0.1:6804/4559",
        "up_thru": 19,
        "heartbeat_back_addrs": {
          "addrvec": [
            {
              "nonce": 4559,
              "type": "legacy",
              "addr": "127.0.0.1:6807"
            }
          ]
        },
        "cluster_addr": "127.0.0.1:6805/4559",
        "osd": 1,
        "uuid": "b6105fad-2e66-4ea3-a0ff-b14eb1f3e52f"
      },
      {
        "public_addrs": {
          "addrvec": [
            {
              "nonce": 4723,
              "type": "legacy",
              "addr": "127.0.0.1:6809"
            }
          ]
        },
        "weight": 1.0,
        "up_from": 19,
        "heartbeat_front_addr": {
          "addrvec": [
            {
              "nonce": 4723,
              "type": "legacy",
              "addr": "127.0.0.1:6811"
            }
          ]
        },
        "down_at": 18,
        "up": 1,
        "cluster_addrs": {
          "addrvec": [
            {
              "nonce": 4723,
              "type": "legacy",
              "addr": "127.0.0.1:6810"
            }
          ]
        },
        "lost_at": 0,
        "primary_affinity": 1.0,
        "state": [
          "exists",
          "up"
        ],
        "last_clean_begin": 11,
        "last_clean_end": 15,
        "in": 1,
        "public_addr": "127.0.0.1:6809/4723",
        "up_thru": 19,
        "heartbeat_back_addrs": {
          "addrvec": [
            {
              "nonce": 4723,
              "type": "legacy",
              "addr": "127.0.0.1:6812"
            }
          ]
        },
        "cluster_addr": "127.0.0.1:6810/4723",
        "osd": 2,
        "uuid": "178b1a4e-1f8d-454e-b2b4-9fccc25e047e"
      }
    ],
    "epoch": 21,
    "require_min_compat_client": "jewel",
    "crush_version": 7,
    "primary_temp": [],
    "pool_max": 2,
    "max_osd": 3,
    "require_osd_release": "nautilus",
    "min_compat_client": "jewel",
    "osd_xinfo": [
      {
        "laggy_probability": 0.0,
        "laggy_interval": 0,
        "features": 4611087854031142911,
        "old_weight": 0,
        "down_stamp": "2018-09-21 10:58:51.931192",
        "osd": 0
      },
      {
        "laggy_probability": 0.0,
        "laggy_interval": 0,
        "features": 4611087854031142911,
        "old_weight": 0,
        "down_stamp": "2018-09-21 10:58:54.178595",
        "osd": 1
      },
      {
        "laggy_probability": 0.0,
        "laggy_interval": 0,
        "features": 4611087854031142911,
        "old_weight": 0,
        "down_stamp": "2018-09-21 10:58:55.191632",
        "osd": 2
      }
    ],
    "blacklist": {
      "127.0.0.1:6815/347750185": "2018-09-22 10:58:55.672857"
    },
    "last_up_change": "2018-09-21 10:58:55.510104",
    "flags_set": [
      "purged_snapdirs",
      "recovery_deletes",
      "sortbitwise"
    ],
    "fsid": "d02162fd-bee9-4cb5-973b-2f7b6d97102a",
    "flags_num": 1605632,
    "created": "2018-09-21 10:41:55.238114",
    "modified": "2018-09-21 10:58:55.672887",
    "flags": "sortbitwise,recovery_deletes,purged_snapdirs",
    "pg_upmap_items": [],
    "pools": [
      {
        "cache_target_full_ratio_micro": 800000,
        "fast_read": false,
        "stripe_width": 0,
        "flags_names": "hashpspool",
        "tier_of": -1,
        "hit_set_grade_decay_rate": 0,
        "pg_placement_num": 8,
        "use_gmt_hitset": true,
        "last_force_op_resend_preluminous": "0",
        "create_time": "2018-09-21 10:42:28.846328",
        "quota_max_bytes": 0,
        "erasure_code_profile": "",
        "snap_seq": 0,
        "expected_num_objects": 0,
        "size": 3,
        "pg_num_pending": 8,
        "auid": 0,
        "cache_min_flush_age": 0,
        "hit_set_period": 0,
        "min_read_recency_for_promote": 0,
        "target_max_objects": 0,
        "pg_placement_num_target": 8,
        "pg_num": 8,
        "type": 1,
        "pg_num_pending_dec_epoch": 12,
        "grade_table": [],
        "pool_name": "cephfs_data_a",
        "cache_min_evict_age": 0,
        "snap_mode": "selfmanaged",
        "pg_num_target": 8,
        "cache_mode": "none",
        "min_size": 1,
        "cache_target_dirty_high_ratio_micro": 600000,
        "object_hash": 2,
        "application_metadata": {
          "cephfs": {
            "data": "cephfs_a"
          }
        },
        "write_tier": -1,
        "cache_target_dirty_ratio_micro": 400000,
        "pool": 1,
        "removed_snaps": "[]",
        "crush_rule": 0,
        "tiers": [],
        "hit_set_params": {
          "type": "none"
        },
        "last_force_op_resend": "0",
        "pool_snaps": [],
        "quota_max_objects": 0,
        "last_force_op_resend_prenautilus": "0",
        "options": {},
        "hit_set_count": 0,
        "flags": 1,
        "target_max_bytes": 0,
        "snap_epoch": 0,
        "hit_set_search_last_n": 0,
        "last_change": "14",
        "min_write_recency_for_promote": 0,
        "read_tier": -1
      },
      {
        "cache_target_full_ratio_micro": 800000,
        "fast_read": false,
        "stripe_width": 0,
        "flags_names": "hashpspool",
        "tier_of": -1,
        "hit_set_grade_decay_rate": 0,
        "pg_placement_num": 8,
        "use_gmt_hitset": true,
        "last_force_op_resend_preluminous": "0",
        "create_time": "2018-09-21 10:42:29.266816",
        "quota_max_bytes": 0,
        "erasure_code_profile": "",
        "snap_seq": 0,
        "expected_num_objects": 0,
        "size": 3,
        "pg_num_pending": 8,
        "auid": 0,
        "cache_min_flush_age": 0,
        "hit_set_period": 0,
        "min_read_recency_for_promote": 0,
        "target_max_objects": 0,
        "pg_placement_num_target": 8,
        "pg_num": 8,
        "type": 1,
        "pg_num_pending_dec_epoch": 13,
        "grade_table": [],
        "pool_name": "cephfs_metadata_a",
        "cache_min_evict_age": 0,
        "snap_mode": "selfmanaged",
        "pg_num_target": 8,
        "cache_mode": "none",
        "min_size": 1,
        "cache_target_dirty_high_ratio_micro": 600000,
        "object_hash": 2,
        "application_metadata": {
          "cephfs": {
            "metadata": "cephfs_a"
          }
        },
        "write_tier": -1,
        "cache_target_dirty_ratio_micro": 400000,
        "pool": 2,
        "removed_snaps": "[]",
        "crush_rule": 0,
        "tiers": [],
        "hit_set_params": {
          "type": "none"
        },
        "last_force_op_resend": "0",
        "pool_snaps": [],
        "quota_max_objects": 0,
        "last_force_op_resend_prenautilus": "0",
        "options": {},
        "hit_set_count": 0,
        "flags": 1,
        "target_max_bytes": 0,
        "snap_epoch": 0,
        "hit_set_search_last_n": 0,
        "last_change": "15",
        "min_write_recency_for_promote": 0,
        "read_tier": -1
      }
    ],
    "new_removed_snaps": [],
    "removed_snaps_queue": []
  },
  "fs_map": {
    "compat": {
      "compat": {},
      "ro_compat": {},
      "incompat": {
        "feature_10": "snaprealm v2",
        "feature_8": "no anchor table",
        "feature_9": "file layout v2",
        "feature_2": "client writeable ranges",
        "feature_3": "default file layouts on dirs",
        "feature_1": "base v0.20",
        "feature_6": "dirfrag is stored in omap",
        "feature_4": "dir inode in separate object",
        "feature_5": "mds uses versioned encoding"
      }
    },
    "feature_flags": {
      "ever_enabled_multiple": false,
      "enable_multiple": false
    },
    "default_fscid": 1,
    "filesystems": [
      {
        "id": 1,
        "mdsmap": {
          "session_autoclose": 300,
          "balancer": "",
          "modified": "2018-09-21 10:58:59.856070",
          "last_failure_osd_epoch": 21,
          "in": [
            0
          ],
          "last_failure": 0,
          "max_file_size": 1099511627776,
          "explicitly_allowed_features": 0,
          "damaged": [],
          "tableserver": 0,
          "failed": [],
          "metadata_pool": 2,
          "epoch": 11,
          "flags": 18,
          "max_mds": 1,
          "compat": {
            "compat": {},
            "ro_compat": {},
            "incompat": {
              "feature_10": "snaprealm v2",
              "feature_8": "no anchor table",
              "feature_9": "file layout v2",
              "feature_2": "client writeable ranges",
              "feature_3": "default file layouts on dirs",
              "feature_1": "base v0.20",
              "feature_6": "dirfrag is stored in omap",
              "feature_4": "dir inode in separate object",
              "feature_5": "mds uses versioned encoding"
            }
          },
          "min_compat_client": "-1 (unspecified)",
          "data_pools": [
            1
          ],
          "info": {
            "gid_34136": {
              "standby_for_rank": -1,
              "addr": "127.0.0.1:6814/2139526630",
              "export_targets": [],
              "name": "b",
              "incarnation": 8,
              "state": "up:active",
              "state_seq": 4,
              "standby_for_fscid": -1,
              "standby_replay": false,
              "gid": 34136,
              "features": 4611087854031142911,
              "rank": 0,
              "standby_for_name": "",
              "addrs": {
                "addrvec": [
                  {
                    "nonce": 2139526630,
                    "type": "legacy",
                    "addr": "127.0.0.1:6814"
                  }
                ]
              }
            }
          },
          "fs_name": "cephfs_a",
          "created": "2018-09-21 10:42:30.305761",
          "standby_count_wanted": 1,
          "enabled": true,
          "up": {
            "mds_0": 34136
          },
          "session_timeout": 60,
          "stopped": [],
          "ever_allowed_features": 0,
          "root": 0
        }
      }
    ],
    "epoch": 11,
    "standbys": [
      {
        "standby_for_rank": -1,
        "addr": "127.0.0.1:6815/1585121408",
        "export_targets": [],
        "name": "c",
        "state": "up:standby",
        "incarnation": 0,
        "epoch": 7,
        "state_seq": 1,
        "standby_for_fscid": -1,
        "standby_replay": false,
        "gid": 34116,
        "features": 4611087854031142911,
        "rank": -1,
        "standby_for_name": "",
        "addrs": {
          "addrvec": [
            {
              "nonce": 1585121408,
              "type": "legacy",
              "addr": "127.0.0.1:6815"
            }
          ]
        }
      },
      {
        "standby_for_rank": -1,
        "addr": "127.0.0.1:6813/3705009905",
        "export_targets": [],
        "name": "a",
        "state": "up:standby",
        "incarnation": 0,
        "epoch": 7,
        "state_seq": 1,
        "standby_for_fscid": -1,
        "standby_replay": false,
        "gid": 34123,
        "features": 4611087854031142911,
        "rank": -1,
        "standby_for_name": "",
        "addrs": {
          "addrvec": [
            {
              "nonce": 3705009905,
              "type": "legacy",
              "addr": "127.0.0.1:6813"
            }
          ]
        }
      }
    ]
  },
  "version": {
    "release": 14,
    "major": 0,
    "full": "ceph version 14.0.0-3517-g5322f99370 (5322f99370d629f6927b9c948522a003fc5da5bb) nautilus (dev)",
    "minor": 0
  },
  "health": {
    "current": {
      "status": "HEALTH_WARN",
      "checks": {
        "PG_AVAILABILITY": {
          "detail": [
            {
              "message": "pg 1.0 is stuck peering for 990.119101, current state peering, last acting [1,0,2]"
            },
            {
              "message": "pg 1.1 is stuck peering for 990.111854, current state peering, last acting [2,0,1]"
            },
            {
              "message": "pg 1.2 is stuck peering for 990.119186, current state peering, last acting [0,2]"
            },
            {
              "message": "pg 1.3 is stuck peering for 990.118224, current state peering, last acting [1,2,0]"
            },
            {
              "message": "pg 1.4 is stuck peering for 990.119116, current state peering, last acting [1,0,2]"
            },
            {
              "message": "pg 1.5 is stuck peering for 990.111987, current state peering, last acting [2,0,1]"
            },
            {
              "message": "pg 1.6 is stuck peering for 990.118940, current state peering, last acting [1,0,2]"
            },
            {
              "message": "pg 1.7 is stuck peering for 990.117449, current state peering, last acting [1,2,0]"
            },
            {
              "message": "pg 2.0 is stuck peering for 989.765896, current state peering, last acting [2,1,0]"
            },
            {
              "message": "pg 2.1 is stuck peering for 989.768816, current state peering, last acting [2,1,0]"
            },
            {
              "message": "pg 2.2 is stuck peering for 989.765611, current state peering, last acting [0,2]"
            },
            {
              "message": "pg 2.3 is stuck peering for 989.768059, current state peering, last acting [1,2,0]"
            },
            {
              "message": "pg 2.4 is stuck peering for 989.750743, current state peering, last acting [1,0,2]"
            },
            {
              "message": "pg 2.5 is stuck peering for 989.768270, current state peering, last acting [1,0,2]"
            },
            {
              "message": "pg 2.6 is stuck peering for 988.783800, current state peering, last acting [0,2]"
            },
            {
              "message": "pg 2.7 is stuck peering for 989.768669, current state peering, last acting [1,0,2]"
            }
          ],
          "severity": "HEALTH_WARN",
          "summary": {
            "message": "Reduced data availability: 16 pgs peering"
          }
        }
      }
    },
    "history": {
      "checks": {
        "OSD_DOWN": {
          "HEALTH_WARN": {
            "detail": [
              "osd.1 (root=default,host=daq) is down",
              "osd.2 (root=default,host=daq) is down"
            ],
            "summary": [
              "1 osds down"
            ]
          }
        },
        "PG_AVAILABILITY": {
          "HEALTH_WARN": {
            "detail": [
              "pg 1.3 is stuck peering for 986.117085, current state peering, last acting [1,2,0]",
              "pg 1.5 is stuck peering for 988.111424, current state peering, last acting [2,0,1]",
              "pg 2.5 is stuck peering for 989.768270, current state peering, last acting [1,0,2]",
              "pg 1.7 is stuck peering for 990.117449, current state peering, last acting [1,2,0]",
              "pg 2.2 is stuck peering for 985.764472, current state peering, last acting [0,2]",
              "pg 1.2 is stuck peering for 988.118624, current state peering, last acting [0,2]",
              "pg 2.6 is stuck peering for 986.783237, current state peering, last acting [0,2]",
              "pg 2.6 is stuck peering for 988.783800, current state peering, last acting [0,2]",
              "pg 1.5 is stuck peering for 990.111987, current state peering, last acting [2,0,1]",
              "pg 1.5 is stuck peering for 986.110848, current state peering, last acting [2,0,1]",
              "pg 2.1 is stuck peering for 989.768816, current state peering, last acting [2,1,0]",
              "pg 1.7 is stuck peering for 986.116310, current state peering, last acting [1,2,0]",
              "pg 2.3 is stuck peering for 985.766920, current state peering, last acting [1,2,0]",
              "pg 2.5 is stuck peering for 987.767707, current state peering, last acting [1,0,2]",
              "pg 1.1 is stuck peering for 990.111854, current state peering, last acting [2,0,1]",
              "pg 2.7 is stuck peering for 985.767530, current state peering, last acting [1,0,2]",
              "pg 2.0 is stuck peering for 985.764757, current state peering, last acting [2,1,0]",
              "pg 2.1 is stuck peering for 987.768253, current state peering, last acting [2,1,0]",
              "pg 2.0 is stuck peering for 989.765896, current state peering, last acting [2,1,0]",
              "pg 1.2 is stuck peering for 990.119186, current state peering, last acting [0,2]",
              "pg 2.6 is stuck peering for 984.782661, current state peering, last acting [0,2]",
              "pg 2.7 is stuck peering for 987.768106, current state peering, last acting [1,0,2]",
              "pg 1.2 is stuck peering for 986.118047, current state peering, last acting [0,2]",
              "pg 2.4 is stuck peering for 987.750181, current state peering, last acting [1,0,2]",
              "pg 1.6 is stuck peering for 986.117801, current state peering, last acting [1,0,2]",
              "pg 1.6 is stuck peering for 988.118377, current state peering, last acting [1,0,2]",
              "pg 2.3 is stuck peering for 987.767497, current state peering, last acting [1,2,0]",
              "pg 2.2 is stuck peering for 987.765049, current state peering, last acting [0,2]",
              "pg 1.0 is stuck peering for 986.117962, current state peering, last acting [1,0,2]",
              "pg 1.4 is stuck peering for 988.118553, current state peering, last acting [1,0,2]",
              "pg 2.5 is stuck peering for 985.767131, current state peering, last acting [1,0,2]",
              "pg 1.0 is stuck peering for 990.119101, current state peering, last acting [1,0,2]",
              "pg 1.0 is stuck peering for 988.118539, current state peering, last acting [1,0,2]",
              "pg 2.4 is stuck peering for 985.749604, current state peering, last acting [1,0,2]",
              "pg 2.3 is stuck peering for 989.768059, current state peering, last acting [1,2,0]",
              "pg 2.2 is stuck peering for 989.765611, current state peering, last acting [0,2]",
              "pg 2.4 is stuck peering for 989.750743, current state peering, last acting [1,0,2]",
              "pg 2.0 is stuck peering for 987.765334, current state peering, last acting [2,1,0]",
              "pg 1.7 is stuck peering for 988.116886, current state peering, last acting [1,2,0]",
              "pg 1.4 is stuck peering for 986.117976, current state peering, last acting [1,0,2]",
              "pg 1.3 is stuck peering for 990.118224, current state peering, last acting [1,2,0]",
              "pg 2.7 is stuck peering for 989.768669, current state peering, last acting [1,0,2]",
              "pg 1.1 is stuck peering for 988.111291, current state peering, last acting [2,0,1]",
              "pg 1.1 is stuck peering for 986.110715, current state peering, last acting [2,0,1]",
              "pg 1.3 is stuck peering for 988.117661, current state peering, last acting [1,2,0]",
              "pg 1.4 is stuck peering for 990.119116, current state peering, last acting [1,0,2]",
              "pg 1.6 is stuck peering for 990.118940, current state peering, last acting [1,0,2]",
              "pg 2.1 is stuck peering for 985.767677, current state peering, last acting [2,1,0]"
            ],
            "summary": [
              "Reduced data availability: 16 pgs peering"
            ]
          }
        },
        "FS_DEGRADED": {
          "HEALTH_WARN": {
            "detail": [
              "fs cephfs_a is degraded"
            ],
            "summary": [
              "1 filesystem is degraded"
            ]
          }
        },
        "MDS_ALL_DOWN": {
          "HEALTH_ERR": {
            "detail": [
              "fs cephfs_a is offline because no MDS is active for it."
            ],
            "summary": [
              "1 filesystem is offline"
            ]
          }
        }
      }
    }
  },
  "manager_map": {
    "available": true,
    "active_change": "2018-09-21 10:58:50.982610",
    "available_modules": [
      {
        "name": "dashboard",
        "error_string": "No module named rbd",
        "can_run": false
      },
      {
        "name": "diskprediction",
        "error_string": "No module named rbd",
        "can_run": false
      },
      {
        "name": "hello",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "influx",
        "error_string": "influxdb python module not found",
        "can_run": false
      },
      {
        "name": "insights",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "iostat",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "localpool",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "orchestrator_cli",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "progress",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "prometheus",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "restful",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "rook",
        "error_string": "Kubernetes module not found",
        "can_run": false
      },
      {
        "name": "selftest",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "smart",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "telegraf",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "telemetry",
        "error_string": "",
        "can_run": true
      },
      {
        "name": "zabbix",
        "error_string": "",
        "can_run": true
      }
    ],
    "modules": [
      "balancer",
      "devicehealth",
      "insights",
      "iostat",
      "restful",
      "status"
    ],
    "active_name": "x",
    "epoch": 10,
    "standbys": [],
    "active_addrs": {
      "addrvec": [
        {
          "nonce": 4254,
          "type": "legacy",
          "addr": "127.0.0.1:6808"
        }
      ]
    },
    "services": {
      "restful": "https://daq:42226/"
    },
    "active_gid": 34106
  },
  "errors": [],
  "mon_map": {
    "features": {
      "optional": [],
      "persistent": [
        "kraken",
        "luminous",
        "mimic",
        "osdmap-prune",
        "nautilus"
      ]
    },
    "created": "2018-09-21 10:41:49.406653",
    "modified": "2018-09-21 10:41:49.406653",
    "epoch": 1,
    "mons": [
      {
        "public_addr": "127.0.0.1:40226/0",
        "name": "a",
        "rank": 0,
        "addr": "127.0.0.1:40226/0"
      },
      {
        "public_addr": "127.0.0.1:40227/0",
        "name": "b",
        "rank": 1,
        "addr": "127.0.0.1:40227/0"
      },
      {
        "public_addr": "127.0.0.1:40228/0",
        "name": "c",
        "rank": 2,
        "addr": "127.0.0.1:40228/0"
      }
    ],
    "fsid": "d02162fd-bee9-4cb5-973b-2f7b6d97102a"
  },
  "config": [
    {
      "name": "mon_pg_warn_min_per_osd",
      "level": "advanced",
      "section": "global",
      "mask": "",
      "value": "3",
      "can_update_at_runtime": true
    },
    {
      "name": "osd_pool_default_min_size",
      "level": "advanced",
      "section": "global",
      "mask": "",
      "value": "1",
      "can_update_at_runtime": true
    },
    {
      "name": "osd_pool_default_size",
      "level": "advanced",
      "section": "global",
      "mask": "",
      "value": "3",
      "can_update_at_runtime": true
    },
    {
      "name": "mon_allow_pool_delete",
      "level": "advanced",
      "section": "mon",
      "mask": "",
      "value": "true",
      "can_update_at_runtime": true
    },
    {
      "name": "mon_data_avail_crit",
      "level": "advanced",
      "section": "mon",
      "mask": "",
      "value": "1",
      "can_update_at_runtime": true
    },
    {
      "name": "mon_data_avail_warn",
      "level": "advanced",
      "section": "mon",
      "mask": "",
      "value": "2",
      "can_update_at_runtime": true
    },
    {
      "name": "mon_osd_reporter_subtree_level",
      "level": "advanced",
      "section": "mon",
      "mask": "",
      "value": "osd",
      "can_update_at_runtime": true
    },
    {
      "name": "mgr/restful/x/server_port",
      "level": "unknown",
      "section": "mgr",
      "mask": "",
      "value": "42226",
      "can_update_at_runtime": false
    },
    {
      "name": "osd_copyfrom_max_chunk",
      "level": "advanced",
      "section": "osd",
      "mask": "",
      "value": "524288",
      "can_update_at_runtime": true
    },
    {
      "name": "osd_debug_misdirected_ops",
      "level": "dev",
      "section": "osd",
      "mask": "",
      "value": "true",
      "can_update_at_runtime": true
    },
    {
      "name": "osd_debug_op_order",
      "level": "dev",
      "section": "osd",
      "mask": "",
      "value": "true",
      "can_update_at_runtime": true
    },
    {
      "name": "osd_scrub_load_threshold",
      "level": "advanced",
      "section": "osd",
      "mask": "",
      "value": "2000.000000",
      "can_update_at_runtime": true
    },
    {
      "name": "mds_debug_auth_pins",
      "level": "dev",
      "section": "mds",
      "mask": "",
      "value": "true",
      "can_update_at_runtime": true
    },
    {
      "name": "mds_debug_frag",
      "level": "dev",
      "section": "mds",
      "mask": "",
      "value": "true",
      "can_update_at_runtime": true
    },
    {
      "name": "mds_debug_subtrees",
      "level": "dev",
      "section": "mds",
      "mask": "",
      "value": "true",
      "can_update_at_runtime": true
    }
  ]
}
"""

DEV_TEST_DATA = """
*** DEVELOPER MODE: setting PATH, PYTHONPATH and LD_LIBRARY_PATH ***
2018-09-26 11:32:02.830 7f39e866c700 -1 WARNING: all dangerous and experimental features are enabled.
2018-09-26 11:32:02.879 7f39e866c700 -1 WARNING: all dangerous and experimental features are enabled.
""" + TEST_DATA


def test_ceph_insights():
    for content in (TEST_DATA, DEV_TEST_DATA):
        ceph = CephInsights(context_wrap(content))

        assert ceph is not None

        assert isinstance(ceph.version, dict)
        assert isinstance(ceph.version["full"], six.string_types)
        for key in ("release", "major", "minor"):
            assert isinstance(ceph.version[key], int)

        assert isinstance(ceph.data["crashes"], dict)
        assert isinstance(ceph.data["health"], dict)
        assert isinstance(ceph.data["config"], list)
        assert isinstance(ceph.data["osd_dump"], dict)
        assert isinstance(ceph.data["df"], dict)
        assert isinstance(ceph.data["osd_tree"], dict)
        assert isinstance(ceph.data["fs_map"], dict)
        assert isinstance(ceph.data["crush_map"], dict)
        assert isinstance(ceph.data["mon_map"], dict)
        assert isinstance(ceph.data["service_map"], dict)
        assert isinstance(ceph.data["manager_map"], dict)
        assert isinstance(ceph.data["mon_status"], dict)
        assert isinstance(ceph.data["pg_summary"], dict)
        assert isinstance(ceph.data["osd_metadata"], dict)
        assert isinstance(ceph.data["version"], dict)
        assert isinstance(ceph.data["errors"], list)


def test_ceph_insights_documentation():
    env = {
        'ceph_insights': CephInsights(context_wrap(TEST_DATA))
    }
    failed, total = doctest.testmod(ceph_insights, globs=env)
    assert failed == 0



"""
Sudoers - files ``/etc/sudoers`` or ``/etc/sudoers.d/*``
========================================================

Module for combining the parsing results of ``/etc/sudoers`` and
``/etc/sudoers.d/*`` files.
"""
from insights import combiner
from insights.parsers.sudoers import SudoersBase, EtcSudoers


@combiner(EtcSudoers)
class Sudoers(SudoersBase):
    """
    Class to combiner the ``/etc/sudoers`` and ``/etc/sudoers.d/*``

    Attributes:
        lines(list): The list of RAW lines of all the ``/etc/sudoers`` and
            ``/etc/sudoers.d/*`` files. The order of lines keeps their original
            order them in files. And the files are read with `"filename`"
            alphabetical order.
        data(dict): The dict of RAW lines of ``/etc/sudoers`` and each file under
            ``/etc/sudoers.d/``, The keys are the filename and values are RAW lines
            of each file.

    .. note::
        1. If there is not `"#includedir /etc/sudoers.d"` line in the entry
           file ``/etc/sudoers``, the ``/etc/sudoers.d/*`` files will be
           skipped.

        2. Two helper functions :func:`insights.parsers.sudoers.SudoersBase.get()`
           and :func:`insights.parsers.sudoers.SudoersBase.last()` are also
           provided to quickly get the specified line(s).
           For details, see the super-class:
           :class:`insights.parsers.sudoers.SudoersBase`.

    Examples:
        >>> type(sudo)
        <class 'insights.combiners.sudoers.Sudoers'>
        >>> sudo.get(['wheel', 'ALL=(ALL)', 'ALL'])
        ['%wheel  ALL=(ALL)       ALL']
        >>> sudo.last("#includedir")
        '#includedir /etc/sudoers.d'
    """
    def __init__(self, sudoers):
        self.lines = []
        self.data = {}
        first = False
        for sdr in sorted(sudoers, key=lambda x: x.file_path):
            self.data[sdr.file_path] = sdr.lines
            self.lines.extend(sdr.lines)
            if not first:
                first = True
                include = sdr.last('#includedir')
                if not include or '/etc/sudoers.d' not in include.split()[-1]:
                    break



import pytest

from insights.combiners.httpd_conf import HttpdConfTree
from insights.combiners.nginx_conf import NginxConfTree
from insights.core.exceptions import SkipComponent
from insights.parsers.httpd_conf import HttpdConf
from insights.parsers.mssql_conf import MsSQLConf
from insights.parsers.nginx_conf import NginxConfPEG
from insights.parsers.rsyslog_conf import RsyslogConf
from insights.combiners.rsyslog_confs import RsyslogAllConf
from insights.specs.datasources.ssl_certificate import (
    httpd_ssl_certificate_files, nginx_ssl_certificate_files,
    mssql_tls_cert_file, httpd_certificate_info_in_nss,
    rsyslog_tls_cert_file
)
from insights.tests import context_wrap


HTTPD_CONF = """
listen 80
listen 443
IncludeOptional "/etc/httpd/conf.d/*.conf"
""".strip()

HTTPD_SSL_CONF = """
<VirtualHost *:443>
  ## SSL directives
  SSLEngine on
  SSLCertificateFile      "/etc/pki/katello/certs/katello-apache.crt"
  SSLCertificateKeyFile   "/etc/pki/katello/private/katello-apache.key"
  SSLCertificateChainFile "/etc/pki/katello/certs/katello-server-ca.crt"
  SSLVerifyClient         optional
  SSLCACertificateFile    "/etc/pki/katello/certs/katello-default-ca.crt"
  SSLVerifyDepth          3
  SSLOptions +StdEnvVars +ExportCertData
</VirtualHost>
""".strip()

HTTPD_SSL_CONF_2 = """
<VirtualHost *:443>
  ## SSL directives
  ServerName a.b.c.com
  SSLEngine on
  SSLCertificateFile      "/etc/pki/katello/certs/katello-apache.crt"
  SSLCertificateKeyFile   "/etc/pki/katello/private/katello-apache.key"
  SSLCertificateChainFile "/etc/pki/katello/certs/katello-server-ca.crt"
  SSLVerifyClient         optional
  SSLCACertificateFile    "/etc/pki/katello/certs/katello-default-ca.crt"
  SSLVerifyDepth          3
  SSLOptions +StdEnvVars +ExportCertData
</VirtualHost>
<VirtualHost *:443>
  ## SSL directives
  ServerName  d.c.e.com
  SSLEngine on
  SSLCertificateFile      "/etc/pki/katello/certs/katello-apache_d.crt"
  SSLCertificateKeyFile   "/etc/pki/katello/private/katello-apache_d.key"
  SSLCertificateChainFile "/etc/pki/katello/certs/katello-server-ca_d.crt"
  SSLVerifyClient         optional
  SSLCACertificateFile    "/etc/pki/katello/certs/katello-default-ca_d.crt"
  SSLVerifyDepth          3
  SSLOptions +StdEnvVars +ExportCertData
</VirtualHost>
<VirtualHost *:443>
  ## SSL directives
  ServerName  f.g.e.com
  SSLEngine off
  SSLCertificateFile      "/etc/pki/katello/certs/katello-apache_e.crt"
  SSLCertificateKeyFile   "/etc/pki/katello/private/katello-apache_e.key"
  SSLCertificateChainFile "/etc/pki/katello/certs/katello-server-ca_e.crt"
  SSLVerifyClient         optional
  SSLCACertificateFile    "/etc/pki/katello/certs/katello-default-ca_e.crt"
  SSLVerifyDepth          3
  SSLOptions +StdEnvVars +ExportCertData
</VirtualHost>
""".strip()

HTTPD_CONF_WITHOUT_SSL = """
<VirtualHost *:80>
    ServerName a.b.c.com
</VirtualHost>
""".strip()

HTTPD_SSL_CONF_NO_VALUE = """
<VirtualHost *:443>
  ## SSL directives
  SSLEngine off
  SSLCertificateFile      ""
  SSLCertificateKeyFile   ""
  SSLCertificateChainFile ""
</VirtualHost>
""".strip()

NGINX_CONF = """
http {
    listen 80;
    listen 443;
    include /etc/nginx/conf.d/*.conf;
}
""".strip()

NGINX_SSL_CONF = """
server {
  ssl_certificate      "/a/b/c.rsa.crt";
  ssl_certificate_key   "/a/b/c.rsa.key";

  ssl_certificate     "/a/b/c.cecdsa.crt";
  ssl_certificate_key "/a/b/c.cecdsa.key";
}
""".strip()

NGINX_SSL_CONF_MULTIPLE_SERVERS = """
server {
    listen          443 ssl;
    server_name     www.example.com;
    ssl_certificate      "/a/b/www.example.com.crt";
    ssl_certificate_key   "/a/b/www.example.com.key";
    ssl_certificate     "/a/b/www.example.com.cecdsa.crt";
    ssl_certificate_key "/a/b/www.example.com.cecdsa.key";
}

server {
    listen          443 ssl;
    server_name     www.example.org;
    ssl_certificate      "/a/b/www.example.org.crt";
    ssl_certificate_key   "/a/b/www.example.org.key";
}
""".strip()

NGINX_CONF_WITHOUT_SSL = """
server {
  server_name 'a.b.c.com';
}
""".strip()

MSSQL_WITH_TLS = """
[sqlagent]
enabled = true
[EULA]
accepteula = Y
[memory]
memorylimitmb = 2048
[network]
tlscert = /tmp/mssql.pem
""".strip()

MSSQL_WITHOUT_TLS = """
[sqlagent]
enabled = true
[EULA]
accepteula = Y
[memory]
memorylimitmb = 2048
""".strip()

HTTPD_NSS_CERT_ENDATE = """
Not After : Mon Jan 18 07:02:43 2038
""".strip()

HTTPD_WITH_NSS = """
Listen 8443
<VirtualHost _default_:8443>
ServerName www.examplea.com:8443
NSSEngine on
NSSCertificateDatabase /etc/httpd/aliasa
NSSNickname testcerta
</VirtualHost>
<VirtualHost :8443>
ServerName www.exampleb.com:8443
NSSEngine on
NSSCertificateDatabase /etc/httpd/aliasb
NSSNickname testcertb
</VirtualHost>
<VirtualHost :8443>
ServerName www.examplec.com:8443
NSSEngine off
NSSCertificateDatabase /etc/httpd/aliasc
NSSNickname testcertc
</VirtualHost>
""".strip()

HTTPD_WITH_NSS_OFF = """
Listen 8443
<VirtualHost _default_:8443>
NSSEngine off
NSSCertificateDatabase /etc/httpd/alias
NSSNickname testcert
</VirtualHost>
""".strip()

RSYSLOG_TLS_CFG1 = """
$ActionQueueFileName abc
$DefaultNetstreamDriver gtls
$DefaultNetstreamDriverCAFile /etc/pki/rsyslog/ca-cert.pem
$DefaultNetstreamDriverCertFile /etc/pki/rsyslog/sender-cert.pem
$DefaultNetstreamDriverKeyFile /etc/pki/rsyslog/sender-key.pem
$ActionSendStreamDriverAuthMode anon
"""

RSYSLOG_TLS_CFG2 = """
global(
workDirectory="/var/lib/rsyslog"
DefaultNetstreamDriverCAFile="/etc/rsyslog.d/certificates/crt/ca.crt"
DefaultNetstreamDriverCertFile="/etc/rsyslog.d/certificates/crt/cert.crt"
DefaultNetstreamDriverKeyFile="/etc/rsyslog.d/certificates/key/driver.key"
)
include(file="/etc/rsyslog.d/*.conf" mode="optional")
*.info;mail.none;authpriv.none;cron.none                /var/log/messages
"""

RSYSLOG_CFG3 = """
global(
workDirectory="/var/lib/rsyslog"
)
include(file="/etc/rsyslog.d/*.conf" mode="optional")
*.info;mail.none;authpriv.none;cron.none                /var/log/messages
"""


def test_httpd_certificate():
    conf1 = HttpdConf(context_wrap(HTTPD_CONF, path='/etc/httpd/conf/httpd.conf'))
    conf2 = HttpdConf(context_wrap(HTTPD_SSL_CONF, path='/etc/httpd/conf.d/ssl.conf'))
    conf_tree = HttpdConfTree([conf1, conf2])

    broker = {
        HttpdConfTree: conf_tree
    }
    result = httpd_ssl_certificate_files(broker)
    assert result == ['/etc/pki/katello/certs/katello-apache.crt']

    conf1 = HttpdConf(context_wrap(HTTPD_CONF, path='/etc/httpd/conf/httpd.conf'))
    conf2 = HttpdConf(context_wrap(HTTPD_SSL_CONF_2, path='/etc/httpd/conf.d/ssl.conf'))
    conf_tree = HttpdConfTree([conf1, conf2])

    broker = {
        HttpdConfTree: conf_tree
    }
    result = httpd_ssl_certificate_files(broker)
    # "/etc/pki/katello/certs/katello-apache_e.crt" not in the result
    assert result == ['/etc/pki/katello/certs/katello-apache.crt', '/etc/pki/katello/certs/katello-apache_d.crt']


def test_nginx_certificate():
    conf1 = NginxConfPEG(context_wrap(NGINX_CONF, path='/etc/nginx/nginx.conf'))
    conf2 = NginxConfPEG(context_wrap(NGINX_SSL_CONF, path='/etc/nginx/conf.d/ssl.conf'))
    conf_tree = NginxConfTree([conf1, conf2])

    broker = {
        NginxConfTree: conf_tree
    }
    result = nginx_ssl_certificate_files(broker)
    assert result == ['/a/b/c.rsa.crt', '/a/b/c.cecdsa.crt']

    conf1 = NginxConfPEG(context_wrap(NGINX_CONF, path='/etc/nginx/nginx.conf'))
    conf2 = NginxConfPEG(context_wrap(NGINX_SSL_CONF_MULTIPLE_SERVERS, path='/etc/nginx/conf.d/ssl.conf'))
    conf_tree = NginxConfTree([conf1, conf2])

    broker = {
        NginxConfTree: conf_tree
    }
    result = nginx_ssl_certificate_files(broker)
    assert result == ['/a/b/www.example.com.crt', '/a/b/www.example.com.cecdsa.crt', '/a/b/www.example.org.crt']


def test_httpd_ssl_cert_exception():
    conf1 = HttpdConf(context_wrap(HTTPD_CONF, path='/etc/httpd/conf/httpd.conf'))
    conf2 = HttpdConf(context_wrap(HTTPD_CONF_WITHOUT_SSL, path='/etc/httpd/conf.d/no_ssl.conf'))
    conf_tree = HttpdConfTree([conf1, conf2])
    broker1 = {
        HttpdConfTree: conf_tree
    }
    conf1 = HttpdConf(context_wrap(HTTPD_CONF, path='/etc/httpd/conf/httpd.conf'))
    conf2 = HttpdConf(context_wrap(HTTPD_SSL_CONF_NO_VALUE, path='/etc/httpd/conf.d/no_ssl.conf'))
    conf_tree = HttpdConfTree([conf1, conf2])
    broker2 = {
        HttpdConfTree: conf_tree
    }
    with pytest.raises(SkipComponent):
        httpd_ssl_certificate_files(broker1)
        httpd_ssl_certificate_files(broker2)


def test_nginx_ssl_cert_exception():
    conf1 = NginxConfPEG(context_wrap(NGINX_CONF, path='/etc/nginx/nginx.conf'))
    conf2 = NginxConfPEG(context_wrap(NGINX_CONF_WITHOUT_SSL, path='/etc/nginx/conf.d/no_ssl.conf'))
    conf_tree = NginxConfTree([conf1, conf2])
    broker1 = {
        NginxConfTree: conf_tree
    }
    with pytest.raises(SkipComponent):
        nginx_ssl_certificate_files(broker1)


def test_mssql_tls_cert_exception():
    conf1 = MsSQLConf(context_wrap(MSSQL_WITH_TLS, path='/var/opt/mssql/mssql.conf'))
    broker1 = {
        MsSQLConf: conf1
    }
    result = mssql_tls_cert_file(broker1)
    assert result == "/tmp/mssql.pem"


def test_mssql_tls_no_cert_exception():
    conf1 = MsSQLConf(context_wrap(MSSQL_WITHOUT_TLS, path='/var/opt/mssql/mssql.conf'))
    broker1 = {
        MsSQLConf: conf1
    }
    with pytest.raises(SkipComponent):
        mssql_tls_cert_file(broker1)


def test_httpd_certificate_info_in_nss():
    conf1 = HttpdConf(context_wrap(HTTPD_CONF, path='/etc/httpd/conf/httpd.conf'))
    conf2 = HttpdConf(context_wrap(HTTPD_WITH_NSS, path='/etc/httpd/conf.d/nss.conf'))
    conf_tree = HttpdConfTree([conf1, conf2])
    broker = {
        HttpdConfTree: conf_tree
    }
    result = httpd_certificate_info_in_nss(broker)
    assert result == [('/etc/httpd/aliasa', 'testcerta'), ('/etc/httpd/aliasb', 'testcertb')]


def test_httpd_certificate_info_in_nss_exception():
    conf1 = HttpdConf(context_wrap(HTTPD_CONF, path='/etc/httpd/conf/httpd.conf'))
    conf2 = HttpdConf(context_wrap(HTTPD_WITH_NSS_OFF, path='/etc/httpd/conf.d/nss.conf'))
    conf_tree = HttpdConfTree([conf1, conf2])
    broker = {
        HttpdConfTree: conf_tree
    }
    with pytest.raises(SkipComponent):
        httpd_certificate_info_in_nss(broker)


def test_rsyslog_certification():
    rsys_par = RsyslogConf(context_wrap(RSYSLOG_TLS_CFG1, path='/etc/rsyslog.conf'))
    rsys_com = RsyslogAllConf([rsys_par])
    broker = {
        RsyslogAllConf: rsys_com
    }
    result = rsyslog_tls_cert_file(broker)
    assert result == '/etc/pki/rsyslog/sender-cert.pem'

    rsys_par = RsyslogConf(context_wrap(RSYSLOG_TLS_CFG2, path='/etc/rsyslog.conf'))
    rsys_com = RsyslogAllConf([rsys_par])
    broker = {
        RsyslogAllConf: rsys_com
    }
    result = rsyslog_tls_cert_file(broker)
    assert result == '/etc/rsyslog.d/certificates/crt/cert.crt'


def test_rsyslog_certification_exception():
    rsys_par = RsyslogConf(context_wrap(RSYSLOG_CFG3, path='/etc/rsyslog.conf'))
    rsys_com = RsyslogAllConf([rsys_par])
    broker = {
        RsyslogAllConf: rsys_com
    }
    with pytest.raises(SkipComponent):
        rsyslog_tls_cert_file(broker)



from insights.combiners.krb5 import AllKrb5Conf
from insights.parsers.krb5 import Krb5Configuration
from insights.tests import context_wrap


KRB5CONFIG = """
# Configuration snippets may be placed in this directory as well
includedir /etc/krb5.conf.d/
include /etc/krb5test.conf
module /etc/krb5test.conf:residual

[libdefaults]
 donotoverwrite1 = true

[logging]
 default = FILE:/var/log/krb5libs.log
 kdc = FILE:/var/log/krb5kdc.log

[realms]
 dns_lookup_realm = false
 default_ccache_name = KEYRING:persistent:%{uid}
 default_ccache_name2 = KEYRING:%{uid}:persistent
 kdc_default_options = default.example.com
 kdc_default_options = default2.example.com
 EXAMPLE.COM = {
  kdc = kerberos.example.com
  admin_server = kerberos.example.com
  auth_to_local = RULE:[1:$1@$0](.*@.*EXAMPLE.ORG)s/@.*//
 }
 EXAMPLE4.COM = {
  kdc = kerberos.example4.com
  admin_server = kerberos.example4.com
 }
 ticket_lifetime = 24h
# renew_lifetime = 7d
# forwardable = true
# rdns = false
""".strip()


KRB5CONFIG2 = """
# Configuration snippets may be placed in this directory as well

[realms]
 dns_lookup_realm = false
 ticket_lifetime = 24h
# default_ccache_name = KEYRING:persistent:%{uid}
 EXAMPLE.COM = {
  kdc = kerberos.example.com
  kdc = test2.example.com
  kdc = test3.example.com
  admin_server = kerberos.example.com
 }
 EXAMPLE2.COM = {
  kdc = kerberos.example2.com
  admin_server = kerberos.example2.com
 }
 EXAMPLE3.COM = {
  kdc = kerberos.example3.com
  admin_server = kerberos.example3.com
 } *

[logging]
 default = FILE:/var/log/krb5libs2.log
 kdc = FILE:/var/log/krb5kdc2.log
 admin_server = FILE:/var/log/kadmind2.log

[libdefaults]
 default_realm = EXAMPLE2.COM
 dns_canonicalize_hostname = false
 donotoverwrite2 = true
 dnsdsd = false
 tilnvs = 24h
 default_ccache_name = KEYRING:%{uid}:persistent
""".strip()


KRB5CONFIG3 = """

[libdefaults]
 donotoverwrite3 = true

""".strip()

KRB5CONFIG4 = """
# This file should normally be installed by your distribution into a
# directory that is included from the Kerberos configuration file (/etc/krb5.conf)
# On Fedora/RHEL/CentOS, this is /etc/krb5.conf.d/

includedir /var/lib/sss/pubconf/krb5.include.d/
""".strip()


def test_active_krb5_nest():
    krb51 = Krb5Configuration(context_wrap(KRB5CONFIG, path='/etc/krb5.conf'))
    krb52 = Krb5Configuration(context_wrap(KRB5CONFIG2, path='/etc/krb5.conf.d/test.conf'))
    krb53 = Krb5Configuration(context_wrap(KRB5CONFIG3, path='/etc/krb5.conf.d/test2.conf'))
    result = AllKrb5Conf([krb51, krb52, krb53])
    assert result.has_option("libdefaults", "donotoverwrite1")
    assert result.has_option("libdefaults", "donotoverwrite2")
    assert result.has_option("libdefaults", "donotoverwrite3")
    assert result.getboolean("libdefaults", "donotoverwrite3") is True
    assert result.getboolean("libdefaults", "dnsdsd") is False
    assert result["logging"]["kdc"] == "FILE:/var/log/krb5kdc.log"
    assert result.has_option("logging", "admin_server")
    assert result["realms"]["EXAMPLE2.COM"]["kdc"] == "kerberos.example2.com"
    assert result["libdefaults"]["default_ccache_name"] == "KEYRING:%{uid}:persistent"
    assert "realms" in result.sections()
    assert "realmstest" not in result.sections()
    assert result.has_section("realms")
    assert not result.has_option("realms", "nosuchoption")
    assert not result.has_option("nosucsection", "nosuchoption")
    assert not result.options("realmsno")
    assert sorted(result.options("logging")) == sorted(['default', 'admin_server', 'kdc'])
    assert result.include == ["/etc/krb5test.conf"]
    assert result.includedir == ["/etc/krb5.conf.d/"]
    assert result.module == ["/etc/krb5test.conf:residual"]
    assert result.files == ['krb5.conf', 'test.conf', 'test2.conf']
    assert result.dns_lookup_realm is True
    assert result.dns_lookup_kdc is True
    assert result.default_realm == "EXAMPLE2.COM"
    assert result.realms == set(["EXAMPLE.COM", "EXAMPLE2.COM", "EXAMPLE3.COM", "EXAMPLE4.COM"])


def test_active_krb5_nest_2():
    krb51 = Krb5Configuration(context_wrap(KRB5CONFIG, path='/etc/krb5.conf'))
    krb52 = Krb5Configuration(context_wrap(KRB5CONFIG4, path='/etc/krb5.conf.d/enable_sssd_conf_dir'))
    result = AllKrb5Conf([krb51, krb52])
    assert result.includedir == ['/etc/krb5.conf.d/', '/var/lib/sss/pubconf/krb5.include.d/']



"""
Components identify system type with regard to virtualization
=============================================================

The ``IsBareMetal`` component in this module is valid if the
:py:class:`insights.combiners.virt_what.VirtWhat` combiner indicates
the host is bare metal.
"""
from insights.combiners.virt_what import VirtWhat
from insights.core.exceptions import SkipComponent
from insights.core.plugins import component


@component(VirtWhat)
class IsBareMetal(object):
    """
    This component uses ``VirtWhat`` combiner to determine the virtualization type.
    It checks if the system is bare metal, otherwise it raises ``SkipComponent``.

    Raises:
        SkipComponent: When system is a virtual machine.
    """
    def __init__(self, virt):
        if virt.is_virtual:
            raise SkipComponent("Not a bare metal system.")



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import net_namespace
from insights.parsers.net_namespace import NetworkNamespace
from insights.tests import context_wrap

LIST_NAMESPACE = """
temp_netns  temp_netns_2  temp_netns_3
""".strip()

LIST_NAMESPACE_2 = """
temp_netns
""".strip()

LIST_NAMESPACE_3 = """
""".strip()

CMD_LIST_NAMESPACE = """
temp_netns_3
temp_netns_2
temp_netns
""".strip()

CMD_LIST_NAMESPACE_2 = """
temp_netns_3
""".strip()

CMD_LIST_NAMESPACE_3 = """
""".strip()


def test_netstat_doc_examples():
    env = {
        'netns_obj': NetworkNamespace(context_wrap(LIST_NAMESPACE))
    }
    failed, total = doctest.testmod(net_namespace, globs=env)
    assert failed == 0


def test_bond_class():
    netns_obj = NetworkNamespace(context_wrap(LIST_NAMESPACE))
    assert netns_obj.netns_list.sort() == ['temp_netns', 'temp_netns_2', 'temp_netns_3'].sort()
    assert len(netns_obj.netns_list) == 3

    netns_obj = NetworkNamespace(context_wrap(LIST_NAMESPACE_2))
    assert netns_obj.netns_list == ['temp_netns']
    assert len(netns_obj.netns_list) == 1

    netns_obj = NetworkNamespace(context_wrap(CMD_LIST_NAMESPACE))
    assert netns_obj.netns_list.sort() == ['temp_netns', 'temp_netns_2', 'temp_netns_3'].sort()
    assert len(netns_obj.netns_list) == 3

    netns_obj = NetworkNamespace(context_wrap(CMD_LIST_NAMESPACE_2))
    assert netns_obj.netns_list == ['temp_netns_3']
    assert len(netns_obj.netns_list) == 1


def test_abnormal():
    with pytest.raises(SkipComponent) as pe:
        NetworkNamespace(context_wrap(LIST_NAMESPACE_3))
    assert "Nothing to parse." in str(pe)

    with pytest.raises(SkipComponent) as pe:
        NetworkNamespace(context_wrap(CMD_LIST_NAMESPACE_3))
    assert "Nothing to parse." in str(pe)



import doctest
import pytest

from insights.parsers import cryptsetup_luksDump
from insights.tests import context_wrap
from insights import SkipComponent


LUKS1_DUMP = """LUKS header information for luks1

Version:       	1
Cipher name:   	aes
Cipher mode:   	xts-plain64
Hash spec:     	sha256
Payload offset:	4096
MK bits:       	512
MK digest:     	db b8 55 5e df 8c c4 b4 b8 0a cc dd 98 b5 d8 64 3a 95 3e 9e 
MK salt:       	1b 16 42 cb 04 3b 77 d8 ff 08 1e 0a 41 68 45 a5 
               	c3 2e 76 04 7b 3f a9 69 9c 9b 51 24 58 47 8d a2 
               	ca fe ba be 7b 3f a9 69 9c 9b 51 24 58 
MK iterations: 	106562
UUID:          	263902da-5f0c-43a9-82eb-cc6f14d90448

Key Slot 0: ENABLED
	Iterations:         	2099250
	Salt:               	3d 99 94 f5 a1 f3 ae cb 4a 3f f0 2d 29 46 22 6b 
	                      	3d 99 94 f5 a1 f3 ae cb 4a 3f f0 2d 29 46 22 6b 
	                      	4d 25 
	Key material offset:	8
	AF stripes:            	4000
Key Slot 1: ENABLED
	Iterations:         	1987820
	Salt:               	27 3c 70 b5 f2 b7 7d f3 29 c2 c8 80 6b ff de cd 
	                      	aa 58 b3 85 9f a1 87 07 c6 4f aa cd 59 28 97 ea 
	                      	aa 58 b3 85 9f a1 87 07 c6 4f aa cd 59 28 97 
	Key material offset:	512
	AF stripes:            	4000
Key Slot 2: ENABLED
	Iterations:         	2052006
	Salt:               	33 28 f2 0c 47 94 e7 40 22 c1 bb 4a 06 aa f0 5c 
	                      	38 cc e4 a8 52 e8 8d 70 b2 1e 9d 47 70 a9 f1 2d 
	Key material offset:	1016
	AF stripes:            	4000
Key Slot 3: DISABLED
Key Slot 4: DISABLED
Key Slot 5: DISABLED
Key Slot 6: DISABLED
Key Slot 7: DISABLED
"""  # noqa

LUKS2_DUMP = """LUKS header information
Version:       	2
Epoch:         	6
Metadata area: 	16384 [bytes]
Keyslots area: 	16744448 [bytes]
UUID:          	cfbcc942-e06b-4c4a-952f-e9c9b2011c27
Label:         	(no label)
Subsystem:     	(no subsystem)
Flags:       	(no flags)

Data segments:
  0: crypt
	offset: 16777216 [bytes]
	length: (whole device)
	cipher: aes-xts-plain64
	sector: 4096 [bytes]

  1: crypt
	offset: 16777216 [bytes]
	length: (whole device)
	cipher: aes-xts-plain64
	sector: 4096 [bytes]
	flags : backup-previous

  2: crypt
	offset: 16777216 [bytes]
	length: (whole device)
	cipher: aes-xts-plain64
	sector: 4096 [bytes]
	flags : backup-final

Keyslots:
  0: luks2
	Key:        512 bits
	Priority:   normal
	Cipher:     aes-xts-plain64
	Cipher key: 512 bits
	PBKDF:      argon2id
	Time cost:  7
	Memory:     1048576
	Threads:    4
	Salt:       3d c4 1b 52 fe 1c 90 d8 2a 35 b2 62 34 e9 0a 59 
	            e9 0e 48 57 b2 dd 45 
	AF stripes: 4000
	AF hash:    sha256
	Area offset:32768 [bytes]
	Area length:258048 [bytes]
	Digest ID:  0
  1: luks2
	Key:        512 bits
	Priority:   normal
	Cipher:     aes-xts-plain64
	Cipher key: 512 bits
	PBKDF:      argon2id
	Time cost:  7
	Memory:     1048576
	Threads:    4
	Salt:       bb b4 33 41 c1 94 15 86 2a e9 26 f8 d8 16 83 b1 
	            d0 00 f9 25 05 2d 80 
	AF stripes: 4000
	AF hash:    sha256
	Area offset:290816 [bytes]
	Area length:258048 [bytes]
	Digest ID:  0
  2: luks2
	Key:        512 bits
	Priority:   normal
	Cipher:     aes-xts-plain64
	Cipher key: 512 bits
	PBKDF:      pbkdf2
	Hash:       sha512
	Iterations: 1000
	Salt:       a8 53 27 a8 d7 8f a6 de a0 cb a4 d1 d4 2c 60 19 
	            f0 32 a2 b0 fb 53 43 
	AF stripes: 4000
	AF hash:    sha512
	Area offset:548864 [bytes]
	Area length:258048 [bytes]
	Digest ID:  0
Tokens:
  0: systemd-tpm2
	Keyslot:    2
Digests:
  0: pbkdf2
	Hash:       sha256
	Iterations: 129774
	Salt:       e6 31 d5 74 e0 65 83 82 35 03 29 56 0e 80 36 5c 
	            4d cd 4d f9 de 69 39 
	Digest:     21 aa b3 dc 9d 46 9b 0f 3a 0f 57 13 80 c6 0b bf 
	            67 66 9e 73 ed 7d 09 
"""  # noqa

LUKS_BAD_DUMP = "Device /dev/nvme0n1p1 is not a valid LUKS device."


def test_cryptsetup_luks_dump_luks1():
    luks1_parsed = cryptsetup_luksDump.LuksDump(context_wrap(LUKS1_DUMP))
    print(luks1_parsed.dump)
    assert luks1_parsed.dump is not None
    assert set(luks1_parsed.dump.keys()) == set(["header"] + ["Key Slot " + str(x) for x in range(8)])
    assert luks1_parsed.dump["header"]["Version"] == "1"
    assert luks1_parsed.dump["Key Slot 0"]["status"] == "ENABLED"
    assert luks1_parsed.dump["Key Slot 7"]["status"] == "DISABLED"


def test_cryptsetup_luks_dump_luks2():
    luks2_parsed = cryptsetup_luksDump.LuksDump(context_wrap(LUKS2_DUMP))
    assert luks2_parsed.dump is not None
    assert set(luks2_parsed.dump.keys()) == set(["header", "Data segments", "Keyslots", "Tokens", "Digests"])
    assert luks2_parsed.dump["header"]["Version"] == "2"
    assert len(luks2_parsed.dump["Keyslots"].keys()) == 3
    assert luks2_parsed.dump["Keyslots"]["0"]["type"] == "luks2"
    assert len(luks2_parsed.dump["Data segments"].keys()) == 3
    assert len(luks2_parsed.dump["Tokens"].keys()) == 1
    assert len(luks2_parsed.dump["Digests"].keys()) == 1

    assert luks2_parsed.dump["Tokens"]["0"]["type"] == "systemd-tpm2"
    assert luks2_parsed.dump["Keyslots"]["0"]["type"] == "luks2"


def test_cryptsetup_luks_dump_bad():
    with pytest.raises(SkipComponent):
        cryptsetup_luksDump.LuksDump(context_wrap(LUKS_BAD_DUMP)).dump
    with pytest.raises(SkipComponent):
        cryptsetup_luksDump.LuksDump(context_wrap("")).dump


def test_doc_examples():
    env = {
            'parsed_result': cryptsetup_luksDump.LuksDump(context_wrap(LUKS2_DUMP)),
          }
    failed, total = doctest.testmod(cryptsetup_luksDump, globs=env)
    assert failed == 0



import os
import json

from collections import defaultdict
from mock.mock import patch, Mock
from pytest import mark

from insights import collect
from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.core import dr
from insights.core import filters
from insights.core.context import HostContext
from insights.core.filters import add_filter
from insights.core.plugins import datasource
from insights.core.spec_cleaner import Cleaner
from insights.core.spec_factory import (
        RawFileProvider, RegistryPoint, SpecSet, command_with_args,
        glob_file, simple_command, simple_file, first_file, foreach_collect)

specs_save_as_manifest = """
---
version: 0

client:
    context:
        class: insights.core.context.HostContext
        args:
            timeout: 5

    # commands and files to ignore
    blacklist:
        files: []
        commands: []
        patterns: []
        keywords: []

    persist:
        - name: insights.tests.test_specs_save_as.Specs
          enabled: true

    run_strategy:
        name: serial
        args:
            max_workers: null

plugins:
    default_component_enabled: false

    packages:
        - insights.tests.test_specs_save_as

    configs:
        - name: insights.tests.test_specs_save_as.Specs
          enabled: true
        - name: insights.tests.test_specs_save_as.Stuff
          enabled: true
""".strip()

SAVE_AS_MAP = {
    'many_glob': ('/many_glob/one.dir', 'many_glob/one.dir/'),
    'many_foreach': ('many_foreach_dir/', 'many_foreach_dir/'),
    'smpl_cmd': ('/smpl_dir/smpl_cmd', 'smpl_dir/smpl_cmd'),
    'smpl_cmd_w_filter': ('smpl_dir_wf/', 'smpl_dir_wf'),
    'cmd_w_args': ('cmd_with_args/', 'cmd_with_args'),
    'smpl_file': (None, None),
    'smpl_file_w_filter': ('smpl_filtered_file/', 'smpl_filtered_file/'),
    'first_file_spec_w_filter': ('/first.file', 'first.file'),
}

here = os.path.abspath(os.path.dirname(__file__))
this_file = os.path.abspath(__file__).rstrip("c")


class Specs(SpecSet):
    many_glob = RegistryPoint(multi_output=True)
    many_foreach = RegistryPoint(multi_output=True)
    smpl_cmd = RegistryPoint()
    smpl_cmd_w_filter = RegistryPoint(filterable=True)
    cmd_w_args = RegistryPoint()
    smpl_file = RegistryPoint()
    smpl_file_w_filter = RegistryPoint(filterable=True)
    first_file_spec_w_filter = RegistryPoint(filterable=True)


class Stuff(Specs):
    many_glob = glob_file(here + "/*.py", save_as=SAVE_AS_MAP['many_glob'][0])

    @datasource(HostContext)
    def ls_files(broker):
        """ Return a file path """
        return this_file

    @datasource(HostContext)
    def files(broker):
        """ Return a list of directories from the spec filter """
        return [here + "/helpers.py", here + "/__init__.py"]

    many_foreach = foreach_collect(files, "%s", save_as=SAVE_AS_MAP['many_foreach'][0])
    smpl_cmd = simple_command("/usr/bin/uptime", save_as=SAVE_AS_MAP['smpl_cmd'][0])
    smpl_cmd_w_filter = simple_command("echo -n ' hello '", save_as=SAVE_AS_MAP['smpl_cmd_w_filter'][0])
    cmd_w_args = command_with_args("ls %s", ls_files, save_as=SAVE_AS_MAP['cmd_w_args'][0])
    smpl_file = simple_file(this_file, save_as=SAVE_AS_MAP['smpl_file'][0])
    smpl_file_w_filter = simple_file(here + "/mock_web_server.py", kind=RawFileProvider, save_as=SAVE_AS_MAP['smpl_file_w_filter'][0])  # RAW file won't filter
    first_file_spec_w_filter = first_file([here + "/spec_tests.py", "/etc/os-release"], save_as=SAVE_AS_MAP['first_file_spec_w_filter'][0])


class stage(dr.ComponentType):
    def invoke(self, broker):
        return self.component(broker)


# File content
with open(this_file) as f:
    smpl_file_content = f.read().splitlines()
with open(here + "/mock_web_server.py", 'rb') as f:  # RawFileProvider: rb; and no filtering
    smpl_file_w_filter_content = f.read()
with open(here + "/spec_tests.py") as f:
    first_file_w_filter_content = [l for l in f.read().splitlines() if any(i in l for i in [" hello ", "class T"])]

#
# TEST
#


def teardown_function(func):
    filters._CACHE = {}

    if func in [test_specs_save_as_collect, test_specs_save_as_no_collect]:
        del filters.FILTERS[Stuff.smpl_cmd_w_filter]
        del filters.FILTERS[Stuff.smpl_file_w_filter]
        del filters.FILTERS[Stuff.first_file_spec_w_filter]

    # Reset Test ENV
    dr.COMPONENTS = defaultdict(lambda: defaultdict(set))
    dr.TYPE_OBSERVERS = defaultdict(set)
    dr.ENABLED = defaultdict(lambda: True)


@stage(Stuff.many_glob,
       Stuff.many_foreach,
       Stuff.smpl_file,
       Stuff.smpl_file_w_filter,
       Stuff.smpl_cmd,
       Stuff.smpl_cmd_w_filter,
       Stuff.cmd_w_args,
       Stuff.first_file_spec_w_filter)
def dostuff(broker):
    assert Stuff.many_glob in broker
    assert Stuff.many_foreach in broker
    assert Stuff.smpl_cmd in broker
    assert Stuff.smpl_cmd_w_filter in broker
    assert Stuff.cmd_w_args in broker
    assert Stuff.smpl_file in broker
    assert Stuff.smpl_file_w_filter in broker
    assert Stuff.first_file_spec_w_filter in broker


def test_specs_save_as_no_collect():
    add_filter(Stuff.smpl_cmd_w_filter, " hello ")
    add_filter(Stuff.smpl_file_w_filter, "def test")
    add_filter(Stuff.first_file_spec_w_filter, [" hello ", "class T"])
    broker = dr.Broker()
    broker[HostContext] = HostContext()
    broker['cleaner'] = Cleaner(None, None)
    broker = dr.run(dr.get_dependency_graph(dostuff), broker)
    # general tests
    assert dostuff in broker, broker.tracebacks
    assert broker[Stuff.smpl_file].content == smpl_file_content
    assert not any(l.endswith("\n") for l in broker[Stuff.smpl_file].content)
    # "filter" works when loading
    assert "hello" in broker[Stuff.smpl_cmd_w_filter].content[0]
    assert len(broker[Stuff.smpl_cmd_w_filter].content) == 1
    assert broker[Stuff.smpl_file_w_filter].content == smpl_file_w_filter_content  # RawFileProvdier only one line
    assert broker[Stuff.first_file_spec_w_filter].content == first_file_w_filter_content
    # test "Save As"
    assert broker[Stuff.many_glob][-1].save_as == SAVE_AS_MAP['many_glob'][1]
    assert broker[Stuff.many_foreach][0].save_as == SAVE_AS_MAP['many_foreach'][1]
    assert broker[Stuff.many_foreach][1].save_as == SAVE_AS_MAP['many_foreach'][1]
    assert broker[Stuff.smpl_cmd].save_as == SAVE_AS_MAP['smpl_cmd'][1]
    assert broker[Stuff.smpl_cmd_w_filter].save_as == SAVE_AS_MAP['smpl_cmd_w_filter'][1]
    assert broker[Stuff.cmd_w_args].save_as == SAVE_AS_MAP['cmd_w_args'][1]
    assert broker[Stuff.smpl_file].save_as == SAVE_AS_MAP['smpl_file'][1]
    assert broker[Stuff.smpl_file_w_filter].save_as == SAVE_AS_MAP['smpl_file_w_filter'][1]
    assert broker[Stuff.first_file_spec_w_filter].save_as == SAVE_AS_MAP['first_file_spec_w_filter'][1]


@mark.parametrize("obfuscate", [True, False])
@patch('insights.core.spec_cleaner.Cleaner.generate_report', Mock())
def test_specs_save_as_collect(obfuscate):
    add_filter(Stuff.smpl_cmd_w_filter, " hello ")
    add_filter(Stuff.smpl_file_w_filter, "def test")
    add_filter(Stuff.first_file_spec_w_filter, [" hello ", "class T"])

    manifest = collect.load_manifest(specs_save_as_manifest)
    for pkg in manifest.get("plugins", {}).get("packages", []):
        dr.load_components(pkg, exclude=None)

    conf = InsightsConfig(
            obfuscate=obfuscate, obfuscate_hostname=obfuscate,
            manifest=specs_save_as_manifest)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()
    output_path, errors = collect.collect(
            tmp_path=arch.tmp_dir,
            archive_name=arch.archive_name,
            client_config=conf)
    meta_data_root = os.path.join(output_path, 'meta_data')
    data_root = os.path.join(output_path, 'data')

    assert not errors
    count = 0
    for spec in Specs.__dict__:
        if not spec.startswith(('__', 'context_handlers', 'registry')):
            file_name = "insights.tests.test_specs_save_as.Specs.{0}.json".format(spec)
            meta_data = os.path.join(meta_data_root, file_name)
            with open(meta_data, 'r') as fp:
                count += 1
                mdata = json.load(fp)
                results = mdata.get('results')
                if not isinstance(results, list):
                    results = [results]
                for result in results:
                    # 1. Checking Save As
                    assert result['object']['save_as'] is bool(SAVE_AS_MAP[spec][1])
                    # relative_path is started with "save_as"
                    rel = result['object']['relative_path']
                    if SAVE_AS_MAP[spec][1] is None:
                        # smpl_file
                        assert rel == this_file.lstrip('/')
                    else:
                        assert SAVE_AS_MAP[spec][1] in rel
                    # 2. Checking collection results in archive
                    # check particular files only
                    if "smpl_file_w_filter" in spec:
                        with open(os.path.join(data_root, rel), 'rb') as fp:
                            assert smpl_file_w_filter_content == fp.read()
                    elif "smpl_file" in spec:
                        with open(os.path.join(data_root, rel), 'r') as fp:
                            new_content = fp.read().splitlines()
                        assert new_content[:-1] == smpl_file_content[:-1]
                        assert new_content[-1].strip() == smpl_file_content[-1].strip()
                    elif "first_file_spec_w_filter" in spec:
                        with open(os.path.join(data_root, rel), 'r') as fp:
                            new_content = fp.read().splitlines()
                        assert new_content == [line for line in first_file_w_filter_content if "class T" in line]
    assert count == len(SAVE_AS_MAP)

    arch.delete_archive_dir()



"""
Collect all the interesting data for analysis
"""
from __future__ import absolute_import
import copy
import errno
import glob
import json
import logging
import os
import shlex
import six

from itertools import chain
from subprocess import Popen, PIPE, STDOUT

from insights.client.archive import InsightsArchive
from insights.client.constants import InsightsConstants as constants
from insights.client.insights_spec import InsightsFile, InsightsCommand
from insights.client.utilities import (_expand_paths, get_version_info,
                                       systemd_notify_init_thread, get_tags)
from insights.core.blacklist import BLACKLISTED_SPECS
from insights.core.dr import get_component_by_name
from insights.core.spec_cleaner import Cleaner

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)


class DataCollector(object):
    '''
    Run commands and collect files
    '''

    def __init__(self, config, archive_=None, mountpoint=None, spec_conf=None):
        self.config = config
        self.archive = archive_ if archive_ else InsightsArchive(config)
        self.mountpoint = mountpoint if mountpoint else '/'
        self.spec_conf = spec_conf if spec_conf else {}

    def _write_branch_info(self, branch_info):
        logger.debug("Writing branch information to archive...")
        self.archive.add_metadata_to_archive(
            json.dumps(branch_info), '/branch_info')

    def _write_display_name(self):
        if self.config.display_name:
            logger.debug("Writing display_name to archive...")
            self.archive.add_metadata_to_archive(
                self.config.display_name, '/display_name')

    def _write_ansible_host(self):
        if self.config.ansible_host:
            logger.debug("Writing ansible_host to archive...")
            self.archive.add_metadata_to_archive(
                self.config.ansible_host, '/ansible_host')

    def _write_version_info(self):
        logger.debug("Writing version information to archive...")
        version_info = get_version_info()
        self.archive.add_metadata_to_archive(
            json.dumps(version_info), '/version_info')

    def _write_tags(self):
        tags = get_tags()
        # NOTE:
        # The following code is also used by datasource 'tags'
        # - insights.specs.datasources.tags
        # Please keep them consistence before removing this.
        if tags is not None:
            def f(k, v):
                if type(v) is list:
                    col = []
                    for val in v:
                        col.append(f(k, val))
                    return list(chain.from_iterable(col))
                elif type(v) is dict:
                    col = []
                    for key, val in v.items():
                        col.append(f(k + ":" + key, val))
                    return list(chain.from_iterable(col))
                else:
                    return [{"key": k, "value": v, "namespace": constants.app_name}]
            logger.debug("Writing tags to archive...")
            t = []
            for k, v in tags.items():
                iv = f(k, v)
                t.append(iv)
            t = list(chain.from_iterable(t))
            self.archive.add_metadata_to_archive(json.dumps(t), '/tags.json')

    def _write_blacklist_report(self, blacklist_report):
        logger.debug("Writing blacklist report to archive...")
        self.archive.add_metadata_to_archive(
            json.dumps(blacklist_report), '/blacklist_report')

    def _write_blacklisted_specs(self):
        if BLACKLISTED_SPECS:
            logger.debug("Writing blacklisted specs to archive...")
            self.archive.add_metadata_to_archive(
                json.dumps({"specs": BLACKLISTED_SPECS}), '/blacklisted_specs')

    def _write_egg_release(self):
        logger.debug("Writing egg release to archive...")
        egg_release = ''
        try:
            with open(constants.egg_release_file) as fil:
                egg_release = fil.read()
        except (IOError, MemoryError) as e:
            logger.debug('Could not read the egg release file: %s', str(e))
        try:
            os.remove(constants.egg_release_file)
        except OSError as e:
            logger.debug('Could not remove the egg release file: %s', str(e))

        try:
            self.archive.add_metadata_to_archive(
                egg_release, '/egg_release')
        except OSError as e:
            logger.debug('Could not add the egg release file to the archive: %s', str(e))
            self.archive.add_metadata_to_archive(
                '', '/egg_release')

    def _write_collection_stats(self, collection_stats):
        logger.debug("Writing collection stats to archive...")
        self.archive.add_metadata_to_archive(
            json.dumps(collection_stats), '/collection_stats')

    def _write_rhsm_facts(self, hashed_fqdn, ip_csv):
        logger.info('Writing RHSM facts to %s ...', constants.rhsm_facts_file)

        hn_block = []
        for k, v in self.hn_db.items():
            hn_block.append({'original': k, 'obfuscated': v})

        kw_block = []
        for k, v in self.kw_db.items():
            kw_block.append({'original': k, 'obfuscated': v})

        ip_block = []
        for k, v in self.ip_db.items():
            ip_block.append(
                {
                    'original': self._int2ip(v),
                    'obfuscated': self._int2ip(k)
                })

        facts = {
            'insights_client.hostname': self.obfuscated_fqdn,
            'insights_client.obfuscate_ip_enabled': 'ip' in self.obfuscate,
            'insights_client.ips': json.dumps(ip_block),
            'insights_client.obfuscate_hostname_enabled': 'hostname' in self.obfuscate,
            'insights_client.hostnames': json.dumps(hn_block),
            'insights_client.keywords': json.dumps(kw_block),
        }

        try:
            with open(constants.rhsm_facts_file, 'w') as fil:
                json.dump(facts, fil)
        except (IOError, OSError) as e:
            logger.error('Could not write to %s: %s', constants.rhsm_facts_file, str(e))

    def _run_pre_command(self, pre_cmd):
        '''
        Run a pre command to get external args for a command
        '''
        logger.debug('Executing pre-command: %s', pre_cmd)
        try:
            pre_proc = Popen(pre_cmd, stdout=PIPE, stderr=STDOUT, shell=True)
        except OSError as err:
            if err.errno == errno.ENOENT:
                logger.debug('Command %s not found', pre_cmd)
            return
        stdout, stderr = pre_proc.communicate()
        the_return_code = pre_proc.poll()
        logger.debug("Pre-command results:")
        logger.debug("STDOUT: %s", stdout)
        logger.debug("STDERR: %s", stderr)
        logger.debug("Return Code: %s", the_return_code)
        if the_return_code != 0:
            return []
        if six.PY3:
            stdout = stdout.decode('utf-8')
        return stdout.splitlines()

    def _parse_file_spec(self, spec):
        '''
        Separate wildcard specs into more specs
        '''
        # separate wildcard specs into more specs
        if '*' in spec['file']:
            expanded_paths = _expand_paths(spec['file'])
            if not expanded_paths:
                return []
            expanded_specs = []
            for p in expanded_paths:
                _spec = copy.copy(spec)
                _spec['file'] = p
                expanded_specs.append(_spec)
            return expanded_specs

        else:
            return [spec]

    def _parse_glob_spec(self, spec):
        '''
        Grab globs of things
        '''
        some_globs = glob.glob(spec['glob'])
        if not some_globs:
            return []
        el_globs = []
        for g in some_globs:
            _spec = copy.copy(spec)
            _spec['file'] = g
            el_globs.append(_spec)
        return el_globs

    def _blacklist_check(self, cmd):
        def _get_nested_parts(cmd):
            parts = shlex.split(cmd.replace(';', ' '))
            all_parts = parts[:]
            for p in parts:
                if len(shlex.split(p)) > 1:
                    all_parts += _get_nested_parts(p)
            return all_parts

        cmd_parts = _get_nested_parts(cmd)
        return len(set.intersection(set(cmd_parts),
                   constants.command_blacklist)) > 0

    def _parse_command_spec(self, spec, precmds):
        '''
        Run pre_commands
        '''
        if 'pre_command' in spec:
            precmd_alias = spec['pre_command']
            try:
                precmd = precmds[precmd_alias]

                if self._blacklist_check(precmd):
                    raise RuntimeError("Command Blacklist: " + precmd)

                args = self._run_pre_command(precmd)
                logger.debug('Pre-command results: %s', args)

                expanded_specs = []
                for arg in args:
                    _spec = copy.copy(spec)
                    _spec['command'] = _spec['command'] + ' ' + arg
                    expanded_specs.append(_spec)
                return expanded_specs
            except LookupError:
                logger.debug('Pre-command %s not found. Skipping %s...',
                             precmd_alias, spec['command'])
                return []
        else:
            return [spec]

    def run_collection(self, rm_conf, branch_info, blacklist_report):
        '''
        Run specs and collect all the data
        '''
        def _get_spec_info(symbolic_name):
            if symbolic_name:
                symbolic_comp = get_component_by_name('insights.specs.Specs.{0}'.format(symbolic_name))
                if symbolic_comp:
                    return symbolic_comp.no_obfuscate, symbolic_comp.no_redact
            return [], False

        # initialize systemd-notify thread
        systemd_notify_init_thread()

        self.archive.create_archive_dir()
        self.archive.create_command_dir()

        collection_stats = {}

        logger.debug('Beginning to run spec collection ...')

        rm_conf = rm_conf or {}
        cleaner = Cleaner(self.config, rm_conf)
        rm_commands = rm_conf.get('commands', [])
        rm_files = rm_conf.get('files', [])

        for c in self.spec_conf['commands']:
            symbolic_name = c.get('symbolic_name')
            command = c.get('command')
            spec_info = _get_spec_info(symbolic_name)
            if command in rm_commands or symbolic_name in rm_commands:
                logger.warn("WARNING: Skipping command %s", command)
                BLACKLISTED_SPECS.append(symbolic_name)
            elif self.mountpoint == "/" or c.get("image"):
                cmd_specs = self._parse_command_spec(c, self.spec_conf['pre_commands'])
                for s in cmd_specs:
                    if s['command'] in rm_commands:
                        logger.warn("WARNING: Skipping command %s", s['command'])
                        BLACKLISTED_SPECS.append(s['symbolic_name'])
                        continue
                    cmd_spec = InsightsCommand(self.config, s, self.mountpoint)
                    self.archive.add_to_archive(cmd_spec, cleaner, spec_info)
                    collection_stats[s['command']] = {
                        'return_code': cmd_spec.return_code,
                        'exec_time': cmd_spec.exec_time,
                        'output_size': cmd_spec.output_size
                    }
        for f in self.spec_conf['files']:
            symbolic_name = f.get('symbolic_name')
            spec_info = _get_spec_info(symbolic_name)
            if f['file'] in rm_files or symbolic_name in rm_files:
                logger.warn("WARNING: Skipping file %s", f['file'])
                BLACKLISTED_SPECS.append(symbolic_name)
            else:
                file_specs = self._parse_file_spec(f)
                for s in file_specs:
                    # filter files post-wildcard parsing
                    if s['file'] in rm_files:
                        logger.warn("WARNING: Skipping file %s", s['file'])
                        BLACKLISTED_SPECS.append(s['symbolic_name'])
                    else:
                        file_spec = InsightsFile(s, self.mountpoint)
                        self.archive.add_to_archive(file_spec, cleaner, spec_info)
                        collection_stats[s['file']] = {
                            'exec_time': file_spec.exec_time,
                            'output_size': file_spec.output_size
                        }
        if 'globs' in self.spec_conf:
            for g in self.spec_conf['globs']:
                symbolic_name = g.get('symbolic_name')
                spec_info = _get_spec_info(symbolic_name)
                if symbolic_name in rm_files:
                    # ignore glob via symbolic name
                    logger.warn("WARNING: Skipping file %s", g['glob'])
                    BLACKLISTED_SPECS.append(symbolic_name)
                else:
                    glob_specs = self._parse_glob_spec(g)
                    for s in glob_specs:
                        if s['file'] in rm_files:
                            logger.warn("WARNING: Skipping file %s", s['file'])
                            BLACKLISTED_SPECS.append(s['symbolic_name'])
                        else:
                            glob_spec = InsightsFile(s, self.mountpoint)
                            self.archive.add_to_archive(glob_spec, cleaner, spec_info)
                            collection_stats[s['file']] = {
                                'exec_time': glob_spec.exec_time,
                                'output_size': glob_spec.output_size
                            }
        cleaner.generate_report(self.archive.archive_name, constants.rhsm_facts_file)
        logger.debug('Spec collection finished.')

        # collect metadata
        logger.debug('Collecting metadata ...')
        self._write_branch_info(branch_info)
        self._write_display_name()
        self._write_ansible_host()
        self._write_version_info()
        self._write_tags()
        self._write_blacklist_report(blacklist_report)
        self._write_blacklisted_specs()
        self._write_egg_release()
        self._write_collection_stats(collection_stats)
        logger.debug('Metadata collection finished.')

    def done(self):
        """
        Do finalization stuff
        """
        if self.config.output_dir:
            return self.archive.archive_dir
        else:
            return self.archive.create_tar_file()
